<?php

/**
 * Configuration functions for the framework.
 *
 * $Id: config.php 610 2006-06-29 10:04:36Z stijn $
 * @package sws
 */

/**
 * The path to the SWS configuration directory, relative to subsite
 * roots.
 */
define('SWSCONFIGDIR', '.sws');

/**
 * The name of the configuration file. Primarily used to override
 * it when running tests.
 */
@define('CONFIGFILE', 'sws');

/**
 * Determine the path to the framework wide configuration files.
 *
 * The return value of this function can be used to prefix framework
 * wide configuration values in cookies or sessions.
 * @return string the absolute filesystem path to the SWS configuration
 *                directory
 */
function configPath()
{
	static $config;

	if (isset($config)) {
		return $config;
	}

	$path = realpath(dirname(__FILE__) . "/../etc");
	if (is_dir($path) && is_readable("$path/" . CONFIGFILE)) {
		$config = $path;
		return $path;
	}

	//XXX: need a proper error mechanism
	print "Could not determine SWS configuration path";
	exit;
}

/**
 * Make sure that a string ends in a slash.
 * @param string $str the string to check
 * @return string the string with a slash appended if necessary
 * @see stripSlash
 */
function addSlash($str)
{
	if (substr($str, -1) != "/") {
		$str = $str . "/";
	}

	return $str;
}

/**
 * Make sure that a string does not end in a slash.
 * @param string $str the string to check
 * @return string the string with the ending slash removed if necessary
 * @see addSlash
 */
function stripSlash($str)
{
	if (substr($str, 0, 1) == "/") {
		return substr($str, 1);
	}

	return $str;
}

/**
 * Parse a configuration file into a hash.
 *
 * The configuration file is expected to look like this:
 *
 * <pre>
 * # Line comments
 * key=1	# number value
 * string="foo"	# string value
 * </pre>
 *
 * This function will return a hash with key 'key', value 1 and
 * key 'string' value "foo" in this case.
 * @param string $file the filename to read
 * @return array hash of configuation values
 */
function parseConfig($file)
{
	$cfg = array();
	$lines = file($file);
	foreach ($lines as $line) {
		$line = trim($line);
		$line = preg_replace('/(^|\s+)#.*/', '', $line);
		$line = preg_replace('/^\s+/', '', $line);
		$line = preg_replace('/\s+$/', '', $line);
		if ($line == "") {
			continue;
		}

		$line = preg_split('/\s*=\s*/', $line, 2);
		if (isset($line[1])) {
			$line[1] = preg_replace('/^"(.*)"$/', '$1', $line[1]);
		} else {
			$line[1] = "1";
		}

		$var = $line[0];
		$cfg[$var] = $line[1];
	}

	return $cfg;
}

/**
 * Determine the correct SWS vhost for a request.
 * @return string the vhost identifier for the current request
 */
function determineVhost()
{
	$config = configPath();
	$host = $_SERVER['SERVER_NAME'];
	$vhost = array_search($host,
	    $_SESSION["conf-$config"]['SWS']['hostnames']);

	// Check for aliases
	if (!$vhost &&
	    isset($_SESSION["conf-$config"]['SWS']['hostname_aliases'])) {
		// Try the list of host_aliases to find the correct virtual host
		foreach ($_SESSION["conf-$config"]['SWS']['hostname_aliases']
		    as $aliashost => $aliases) {
			foreach($aliases as $alias) {
				if (preg_match("/^" . $alias . "$/", $host)) {
					$vhost = $aliashost;
				}
			}
		}
	}

	// Use the default vhost if no match was found
	if (!$vhost) {
		$vhost = "_default_";
	}

	return $vhost;
}


/**
 * Returns the computed base URL for a subsite.
 *
 * This function tries to determine the base URL for a subsite, in case
 * no base URL was configured. This is done by repeatedly stripping
 * pathname components from both the URL and the request directory, in
 * order to find the common request basis.
 *
 * Currently, in the following cases, the base URL can be computed.
 *
 * Site definition	Request directory	Request URL	Base URL
 * /site1		/site1			/		/
 * /site1		/site1/foo		/foo/		/
 * /site1/site2		/site1/site2		/site2/		/site2/
 * /site1/site2		/site1/site2/bar	/site2/bar/	/site2/
 * /subsites/[^/]+	/subsites/site3		/site3/		/site3/
 * /subsites/[^/]+	/subsites/site3/baz	/site3/baz/	/site3/
 * /subsites/[^/]+/s	/subsites/site3/s	/site3/s/	/site3/s/
 * /subsites/[^/]+/s	/subsites/site3/s/2	/site3/s/2/	/site3/s/
 * /foo			/foo			/xyzzy/		/xyzzy/
 * /foo			/foo/bar		/xyzzy/bar/	/xyzzy/
 *
 * The function will not be able to determine base URLs in case of multiple
 * Alias'es. Those subsites need to explicitly define the base URL in their
 * configuration file.
 * @param string $site the site definition for the current site
 * @param string $dir the directory of the current request
 * @param string $request the current request directory
 * @return string the found base URL, or "" if none was found
 */
function findBaseURL($site, $dir, $request)
{
	if (preg_match("#^(" . $site . ")$#", $dir)) {
		return $request;
	}

	$extra = substr($dir, strlen($site));
	if (substr($extra, -1) == "/") {
		$extra = substr($extra, 0, -1);
	}

	$check = array_reverse(explode("/", $extra));
	foreach ($check as $component) {
		if ($component == "") {
			// We're up to the leading slash, exit
			break;
		}

		$c = substr($request, strlen($request) - strlen($component) - 1, -1);
		if ($c != $component) {
			break;
		}

		$request = substr($request, 0, strlen($request) - strlen($component) - 1);
		$dir = substr($dir, 0, strlen($dir) - strlen($component) - 1);
	}

	if (preg_match("#^(" . $site . ")$#", $dir)) {
		return $request;
	}

	// Hardcode one special common case: the login code, which is
	// situated below the SWS main root, which normally is a direct
	// path below a main site of a vhost. So, check to see if
	// the requested URL matches the SWS login URL, and if so
	// use the main site of the current host as a site.
	$config = configPath();
	print "DEBUG: host " . $_SERVER['SERVER_NAME'] . " dir " . $dir . " request " . $request . "<br />\n";
	$loginurl = addSlash($_SESSION["conf-$config"]['SWS']['swsurl']) . "login";
	if (substr($request, 0, strlen($loginurl)) == $loginurl) {
		// This is a request for the login URL; assume we are coming
		// from the main site of this host. This is not always correct
		// but if so it's easily overridden by setting baseurl in
		// a configuration file; this code makes sure that by default
		// we won't need configuration files for regular subsites.
		$vhost = determineVhost();
		$vhost_cfg = $_SESSION["conf-$config"]['SWS']['vhosts'][$vhost];
		return $vhost_cfg['mainsite'];
	}

	// We didn't find anything, unfortunately.
	return "";
}

/**
 * Update the site configuration hash with the correct include directories.
 *
 * Not meant to be called by anthing else but {@link makeSiteConfig}.
 * @param array $vhost_cfg current vhost configuration hash
 * @param array $site_cfg current site configuration hash
 * @param string $site the current subsite filesystem path
 * @return array the updated site configuration hash
 * @access private
 */
function makeSiteDirs($vhost_cfg, $site_cfg, $site)
{
	$subdirs = array(
	    "box" => "boxes",
	    "ext" => "extensions",
	    "theme" => "themes"
	);

	$baseurl = $site_cfg['baseurl'];

	// Generate the urls/dirs based on the sws root url (if set)
	$config = configPath();
	if (isset($_SESSION["conf-$config"]['SWS']['swsurl']) &&
	    $_SESSION["conf-$config"]['SWS']['swsurl'] != "") {
		foreach (array_keys($subdirs) as $prefix) {
			$site_cfg[$prefix . 'urls'][] =
			    addSlash($_SESSION["conf-$config"]['SWS']['swsurl'])
				. addSlash($subdirs[$prefix]);
			$site_cfg[$prefix . 'dirs'][] =
			   addSlash(swsroot()) . addSlash($subdirs[$prefix]);
		}
	}

	// Generate the urls/dirs based on the vhost config
	foreach (array_keys($subdirs) as $prefix) {
		$site_cfg[$prefix . 'urls'][] =
		    addSlash($baseurl) . addSlash(SWSCONFIGDIR) .
			addSlash($subdirs[$prefix]);
		$site_cfg[$prefix . 'dirs'][] =
		    addSlash($site) . addSlash(SWSCONFIGDIR) .
			addSlash($subdirs[$prefix]);
	}

	// For subsites, also include the main site urls/dirs (so that they
	// don't need a separate copy of the themes/boxes/extensions).
	if (isset($vhost_cfg['mainsite']) &&
	    $vhost_cfg['mainsite'] != $site) {
		$mainsite = $vhost_cfg['mainsite'];
		$mainurl = $vhost_cfg['mainurl'];
		foreach (array_keys($subdirs) as $prefix) {
			$site_cfg[$prefix . 'urls'][] =
			    addSlash($mainurl) . addSlash(SWSCONFIGDIR) .
				addSlash($subdirs[$prefix]);
			$site_cfg[$prefix . 'dirs'][] =
			    addSlash($mainsite) . addSlash(SWSCONFIGDIR) .
				addSlash($subdirs[$prefix]);
		}
	}

	return $site_cfg;
}

/**
 * Create the configuration hash for a vhost/subsite combination.
 *
 * This function is used internally by {@link setConfig} at the first
 * request for a subsite, and sets all session parameters for that
 * subsite.
 * @param array $vhost_cfg the current vhost configuration hash
 * @param string $site the filesystem path to the site
 * @return array the site configuration hash
 * @access private
 */
function makeSiteConfig($vhost_cfg, $site)
{
	// Check to see if there is an explicit configuration file
	if (is_readable(addSlash($site) . addSlash(SWSCONFIGDIR) . "config")) {
		$site_cfg = parseConfig(addSlash($site) .
		    addSlash(SWSCONFIGDIR) . "config");
	}

	// If no base URL was specified, try to guess. If that doesn't work,
	// bail out, because otherwise local themes will not work.
	if (!isset($site_cfg['baseurl']) || $site_cfg['baseurl'] == "") {
		$url = normalizeURL($_SERVER['REQUEST_URI']);
		$site_cfg['baseurl'] = findBaseURL(addSlash($site),
		    addSlash(dirname($_SERVER['SCRIPT_FILENAME'])),
		    addSlash(dirname($url)));

		if ($site_cfg['baseurl'] == "") {
			//XXX: better error mechanism
			print "SWS: could not figure out base URL for this " .
			    "subsite, please define it manually in " .
			    addSlash($site) . addSlash(SWSCONFIGDIR) .
			    "config\n";
			exit;
		}
	}

	// Sanitize the configuration
	$site_cfg['baseurl'] = addSlash($site_cfg['baseurl']);
	$site_cfg['basedir'] = $site;

	if (!isset($site_cfg['defaulttheme'])) {
		$site_cfg['defaulttheme'] = $vhost_cfg['defaulttheme'];
	}
	if (!isset($site_cfg['debug'])) {
		$site_cfg['debug'] = $vhost_cfg['debug'];
	}
	$site_cfg['hostname'] = $vhost_cfg['hostname'];
	$site_cfg['https'] = $vhost_cfg['https'];
	$site_cfg['allow_insecure_login'] = $vhost_cfg['allow_insecure_login'];
	$site_cfg = makeSiteDirs($vhost_cfg, $site_cfg, $site);

	return $site_cfg;
}

/**
 * Create the configuration hash for one virtual host.
 * @param string $vhost the name of the virtual host to configure
 * @return array the configuration hash for the virtual host
 * @access private
 */
function readVhostConfig($vhost)
{
	$config = configPath();
	if ($vhost == "_default_") {
		$vhost_cfg = $_SESSION["conf-$config"]['SWS'];
	} else {
		$config = configPath();
		$vhost_cfg = parseConfig($config . "/vhosts/" . $vhost);
	}

	// We always configure the main site
	$sites[] = $vhost_cfg['mainsite'];

	// Now collect all subsite paths
	$subsites = explode(",", $vhost_cfg['subsites']);
	foreach ($subsites as $site) {
		if ($site != "") {
			$site = str_replace('$mainsite',
			    $vhost_cfg['mainsite'], $site);
			$sites[] = $site;
		}
	}

	// Make sure to sort them in reverse order, to find more detailed
	// subsites first
	rsort($sites, SORT_STRING);
	$vhost_cfg['sites'] = $sites;

	// Add this vhost to the list of available hostnames
	$_SESSION["conf-$config"]['SWS']['hostnames'][$vhost] =
	    $vhost_cfg['hostname'];

	if (isset($vhost_cfg['hostname_aliases'])) {
		// Add this vhost's aliases to the list of available
		// hostname_aliases
		$_SESSION["conf-$config"]['SWS']['hostname_aliases'][$vhost] =
		    explode(",", $vhost_cfg['hostname_aliases']);
	}

	// Don't set main parameters in the default host config space
	if ($vhost == "_default_") {
		unset($vhost_cfg['version']);
		unset($vhost_cfg['swsroot']);
		unset($vhost_cfg['swsurl']);
		unset($vhost_cfg['hosts']);
	}

	// Done, now add this configuration map to the session configuration
	$_SESSION["conf-$config"]['SWS']['vhosts'][$vhost] = $vhost_cfg;
}

/**
 * Read the complete SWS configuration and construct all configuration hashes.
 *
 * This is only meant to be called once per user session, by {@link setConfig}.
 * @access private
 */
function readConfig()
{
	$config = configPath();
	$_SESSION["conf-$config"]['SWS'] = parseConfig("$config/" . CONFIGFILE);

	if (!$_SESSION["conf-$config"]['SWS'] ||
	    !$_SESSION["conf-$config"]['SWS']['version']) {
		print "Could not determine SWS configuration version\n";
		exit;
	}

	if ($_SESSION["conf-$config"]['SWS']['version'] != SWSVERSION) {
		print "Wrong version of SWS configuration, expecting " .
		    SWSVERSION  . " got " .
		    $_SESSION["conf-$config"]['SWS']['version'];
		exit;
	}

	if ($_SESSION["conf-$config"]['SWS']['logindomain'] == '') {
		$_SESSION["conf-$config"]['SWS']['logindomain'] =
		    $_SESSION["conf-$config"]['SWS']['hostname'];
	}

	// Read the configuration for each vhost
	$vhosts = array();
	$cfgvhosts = explode(",", $_SESSION["conf-$config"]['SWS']['hosts']);
	foreach ($cfgvhosts as $vhost) {
		if ($vhost != "") {
			if (!is_readable($config . "/vhosts/" . $vhost)) {
				print "Host configuration not " .
				    "readable for $vhost -- ignored";
			} else {
				$vhosts[] = $vhost;
			}
		}
	}
	array_unshift($vhosts, "_default_");
	unset($_SESSION["conf-$config"]['SWS']['hosts']);

	foreach ($vhosts as $vhost) {
		readVhostConfig($vhost);
	}

	// Done, set the 'config read' variable so that we won't do this again
	// until the configuration file is changed.
	$_SESSION["conf-$config"]['SWS']['read'] = filemtime("$config/" . CONFIGFILE);
}

/**
 * Set the configuration for the current request to that for (vhost, site).
 *
 * @access private
 */
function useConfig($vhost, $site)
{
	$config = configPath();
	$vhost_cfg = $_SESSION["conf-$config"]['SWS']['vhosts'][$vhost];

	// Determine whether the configuration file for ($vhost, $site) has
	// been changed since we last read it.
	$cfg = addSlash($site) . addSlash(SWSCONFIGDIR) . "config";
	if (is_readable($cfg)) {
		$mt = filemtime($cfg);
	} else {
		$mt = 0;
	}

	// Check if we had a request for this site in this session yet,
	// or if the constructed configuration is superceded by a newer
	// configuration file.
	if (!isset($vhost_cfg[$site]) || $vhost_cfg[$site]['read'] != $mt) {
		$_SESSION["conf-$config"]['SWS']['vhosts'][$vhost][$site] =
			makeSiteConfig($vhost_cfg, $site);
		$_SESSION["conf-$config"]['SWS']['vhosts'][$vhost][$site]['read'] = $mt;

		// Reset this variable to make sure we have the updated settings
		$vhost_cfg = $_SESSION["conf-$config"]['SWS']['vhosts'][$vhost];
	}

	// Now set the right parameters, and then we're done.
	$_SESSION["conf-$config"]['sws'] = $vhost_cfg[$site];
	$_SESSION["conf-$config"]['sws']['host'] = $vhost;
	$_SESSION["conf-$config"]['sws']['site'] = $site;
	return;
}

/**
 * Process the configuration for a specific page request.
 *
 * This function is called on every page view to set the configuration
 * variables for that request. It makes sure that
 * $_SESSION["conf-$config"]['sws'] points to the correct configuration
 * hash for the requested vhost/site combination.
 *
 * The static configuration for the framework install is read on startup
 * and everytime we detect that the main configuration file has changed.
 * Note that this means that changes in the vhost configuration files will
 * <b>NOT</b> be detected unless a user restarts the browser, or the main
 * configuration file is rewritten also.
 *
 * Notable keys in the $_SESSION["conf-$config"]['sws'] configuration hash:
 * - host: the current virtual host
 * - basedir: the filesystem path to the current site
 * - baseurl: the URL path to the current site
 * - debug: 1 if debugging is enabled, 0 otherwise
 */
function setConfig()
{
	$config = configPath();
	if (!isset($_SESSION["conf-$config"]) ||
	    !isset($_SESSION["conf-$config"]['SWS']) ||
	    !isset($_SESSION["conf-$config"]['SWS']['read']) ||
	    $_SESSION["conf-$config"]['SWS']['read'] != filemtime("$config/" . CONFIGFILE)) {
		readConfig();
		unset($_SESSION["user-$config"]);
	}

	// Determine the virtual host for this request
	$vhost = determineVhost();
	$vhost_cfg = $_SESSION["conf-$config"]['SWS']['vhosts'][$vhost];

	// Check for the current script path
	if (!isset($_SERVER['SCRIPT_FILENAME'])) {
		// Should not happen, but don't error out if it does;
		// just use the main site data
		useConfig($vhost, $vhost_cfg['mainsite']);
		return;
	}

	// First determine the site based on filesystem path to the script
	$fn = dirname($_SERVER['SCRIPT_FILENAME']);
	$sites = $vhost_cfg['sites'];
	foreach ($sites as $site) {
		if (strpos($fn, $site) === 0) {
			useConfig($vhost, $site);
			return;
		}
	}

	// We did not have a match based on straight path. Check for a
	// match based on regexp.
	$match = "";
	foreach ($sites as $site) {
		if (preg_match("#^(" . $site . ")#", $fn, $capt) == 1) {
			// We got a match with a regexp; clone this
			// configuration for the base subsite
			$match = $capt[0];
			break;
		}
	}

	// If we still don't have a match, assume that this is a mapped
	// path in the main site.
	if ($match == "") {
		useConfig($vhost, $vhost_cfg['mainsite']);
		return;
	}

	// It was a match, use the configuration for that one
	useConfig($vhost, $match);
}

?>
