<?php

/**
 * Database functions and classes.
 *
 * $Id: database.php 594 2006-03-20 11:12:42Z stijn $
 * @package sws
 */

/**#@+
 * <p>Status code returned by {@link db_mysql::connect} and
 * {@link db_postgresl::connect} when
 */
/**
 * Database connection ok status code.
 *
 * the database connection was succesfully created.</p>
 */
define('DB_OK', 1);
/**
 * Database type not supported status code.
 *
 * the configured database was of a type that was not supported by the
 * current installation of PHP.</p>
 */
define('DB_NOT_SUPPORTED', -1);
/**
 * Database connection failed status code.
 *
 * the database connection could not be made; an invalid combination of
 * databasename/user/password was given or the database user lacks
 * permissions.</p>
 */
define('DB_NOT_CONNECTED', -2);
/**#@-*/

/* These must correspond to the various {MYSQL,PGSQL}_* constants! */
/**#@+
 * <p>Method indicator for {@link db_mysql::fetch_array} and
 * {@link db_postgresql::fetch_array}. The resulting array will be
 * returned
 */
/**
 * Use only associative arrays for query result sets.
 *
 * as a purely associative array.</p>
 */
define('FETCH_ASSOC', 1);
/**
 * Use a numerically indexed array for query result sets.
 *
 * as a purely numerically indexed array.</p>
 */
define('FETCH_NUM', 2);
/**
 * Use both associative and numerical indices for query result sets.
 *
 * as an array wich both numerically and associatively indexes the
 * rows of the result set.</p>
 */
define('FETCH_BOTH', 3);
/**#@-*/

/**
 * Next row value.
 *
 * Indicates that the caller wants to fetch the next row to
 * {@link db_mysql::fetch_array} or {@link db_postgresql::fetch_array}.
 */
define('FETCH_NEXTROW', -1);

/**
 * Retrieve configured database parameters for a database.
 *
 * This function returns an associative array of parameters for a specific
 * database, as configured by the system.
 *
 * Configuration files are searched for in SWSCONFIGDIR/db, and then in
 * $swsrootdir/etc/db, in that order.
 *
 * The following keys are present in the resulting array:
 *
 * - type, the type of database (mysql, postgresql)
 * - host, the database server hostname
 * - name, the name of the database
 * - user, the database user name
 * - password, the database user password
 *
 * @param string $dbname the name of the database configuration file
 * @return array an associative array with database parameters.
 */
function db_get_parameters($dbname)
{
	$config = configPath();

	$dirs = array();
	if (isset($_SESSION["conf-$config"]['sws']['basedir'])
	    && $_SESSION["conf-$config"]['sws']['basedir'] != "") {
		array_push($dirs,
		    addSlash($_SESSION["conf-$config"]['sws']['basedir']) .
		    addSlash(SWSCONFIGDIR) . "db/");
	}
	array_push($dirs, addSlash($config) . "db/");
	$found = 0;
	foreach ($dirs as $dir) {
		if (is_file($dir . $dbname)) {
			if (!is_readable($dir . $dbname)) {
				//XXX: better error mechanism...
				print "Database parameter file " .
				    $dir . $dbname . " not readable.\n";
				exit;
			}
			$found = 1;
			$dbparm = parseConfig($dir . $dbname);
			break;
		}
	}
	if (!$found) {
		print "Could not find database parameter " .
		    "file for database $dbname in path " .
		    implode(":", $dirs) . "\n";
		exit;
	}

	return $dbparm;
}

/**
 * Handle the logistics of asking for a database user/password using a
 * login form.
 *
 * This function is called internally by {@link db_get_object} when
 * a database configuration file contains the string '__ASK__' as a
 * username. It then presents a login form to ask the user for the
 * needed database credentials.
 *
 * Upon succesfull authentication by the user, it returns an array
 * with member 0 being the resource result for the database, and
 * member 1 being the appropriate database object (either {@link db_mysql}
 * or {@link db_postgresql}).
 *
 * Note that due to the use of redirection, this function should not
 * be called when a page has already output anything to the browser.
 *
 * @param string $dbname the name of the database for wich authentication is needed
 * @return array an array of (resource, database object) for a connected database
 * @access private
 */
function db_ask_password($dbname)
{
	if (isset($_COOKIE['SWSdbs'])) {
		if (ini_get("magic_quotes_gpc") == 1) {
			$dbs = unserialize(stripslashes($_COOKIE['SWSdbs']));
		} else {
			$dbs = unserialize($_COOKIE['SWSdbs']);
		}
		if (isset($dbs[$dbname])) {
			$param = db_get_parameters($dbname);
			$class = "db_" . $param['type'];
			$db = new $class;
			$res = $db->connect($param['host'],
			    $dbs[$dbname]['user'],
			    str_rot13($dbs[$dbname]['password']),
			    $param['name']);
			if ($res == DB_OK) {
				return array($res, $db);
			}
		}
	} else {
		$dbs = array();
	}

	encryptProtocol();

	if (!isset($_SESSION['doingdblogin'])) {
		$_SESSION['doingdblogin'] = 1;
		$_SESSION['saveddbpost'] = serialize($_POST);
		$_SESSION['saveddbget'] = serialize($_GET);
		$_SESSION['saveddbquery'] = $_SERVER['QUERY_STRING'];
		$_SESSION['saveddbrequest'] = serialize($_REQUEST);
		needTheme();
		loginForm(LOGIN_REQUIRED, $dbname);
		exit;
	}

	$user = isset($_POST['user']) ? $_POST['user'] : "";
	$password = isset($_POST['password']) ? $_POST['password'] : "";
	$param = db_get_parameters($dbname);
	$class = "db_" . $param['type'];
	$db = new $class;
	$res = $db->connect($param['host'],
	    $user, $password, $param['name']);
	if ($res == DB_NOT_CONNECTED || $res == DB_NOT_SUPPORTED) {
		needTheme();
		loginForm(LOGIN_FAILED, $dbname);
		exit;
	}

	if (headers_sent($file, $line) && debugging_on()) {
		print "Warning: cannot set cookie for database " .
		    "because the HTTP headers are already sent " .
		    "(by output from " . $file . ":" . $line . ")\n";
	} else {
		$dbs[$dbname] = array(
			'user' => $user,
			'password' => str_rot13($password)
		);
		setcookie('SWSdbs', serialize($dbs));
	}

	unset($_SESSION['doingdblogin']);
	$_POST = unserialize($_SESSION['saveddbpost']);
	unset($_SESSION['saveddbpost']);
	$_GET = unserialize($_SESSION['saveddbget']);
	unset($_SESSION['saveddbget']);
	$_REQUEST = unserialize($_SESSION['saveddbrequest']);
	unset($_SESSION['saveddbrequest']);
	$_SERVER['QUERY_STRING'] = $_SESSION['saveddbquery'];

	return array($res, $db);
}

/**
 * Return a connected database object.
 *
 * This is the master function to connect to a database. Based on a database
 * name, it returns either an error code or a connected database object.
 *
 * The database name is actually the name of an SWS database configuration
 * file, in which the actual parameters for connecting to the database
 * are defined. See db_get_parameters for details about the configuration
 * file location. See $swsroot/etc/db.example for an example of the syntax.
 *
 * This function caches the connection per request. The first call will
 * initiate the connection to the database and the connection will persist
 * until the request has been completed or the {@link db_mysql::close}
 * method is called to close the connection explicitly. Note that in the
 * last case, it is not possible to reconnect to the same database on the
 * same request!
 *
 * @param string $dbname database alias to connect to
 * @return object the connected database object
 */
function db_get_object($dbname)
{
	static $conns = array();

	if (isset($conns[$dbname])) {
		return $conns[$dbname];
	}

	$param = db_get_parameters($dbname);
	if (isset($param['password']) &&
	    $param['password'] == "__ASK__") {
		list($res, $conns[$dbname]) = db_ask_password($dbname);
	} else {
		$class = "db_" . $param['type'];
		$conns[$dbname] = new $class;
		$res = $conns[$dbname]->connect($param['host'],
		    $param['user'], (isset($param['password']) ?
			$param['password'] : ''), $param['name']);
	}

	if ($res == DB_OK) {
		return $conns[$dbname];
	}

	if (debugging_on()) {
		switch ($res) {
		case DB_NOT_SUPPORTED:
			print "Error: Your PHP installation does not " .
			    "include support for databases of type '" .
			    $param['type'] . "'.\n";
			break;
		case DB_NOT_CONNECTED:
			print "Error: connection to database " .
			    $param['name'] . " at host " .
			    $param['host'] . " for user " .
			    $param['user'] . " failed.\n";
			break;
		default:
			print "Error: an unknown error with the " .
			    "database connection occurred ($res).\n";
			break;
		}
	} else {
		print "A database error occurred.\n";
	}

	exit;
}

/**
 * Returns a privileged connection to the SWS database. Obviously only
 * works if the caller is allowed access (this might not be the case in
 * a secure virtual host setup).
 * @return object $db a database object representing the SWS database, or FALSE if access was denied
 */
function db_get_sws_privileged_object()
{
	$config = configPath();
	$dbdir = swsroot() . "/etc/db/";
	if (file_exists($dbdir . "swspriv")) {
		$name = "swspriv";
	} else {
		$name = "sws";
	}

	if (!is_readable($dbdir . $name)) {
		return FALSE;
	}

	return db_get_object($name);
}

?>
