<?php

/**
 * SWS abstract database base class.
 *
 * $Id$
 * @package sws
 */

/**
 * Class representing a query result for a database.
 * @package sws
 */
abstract class db_result
{
	protected $conn;
	protected $query;
	protected $ign;
	protected $aff;

	/**
	 * Initializes a result class.
	 *
	 * Constructor for the class. This class is only meant to be
	 * instantiated by the appropriate {@link db::query}.
	 * @access private
	 */
	function __construct($conn, $query, $ign)
	{
		$this->conn = $conn;
		$this->query = $query;
		$this->ign = $ign;
	}

	/**
	 * Returns the number of fields per row in the result set.
	 * @return int
	 */
	abstract function num_fields();

	/**
	 * Returns the number of rows in the result set.
	 * @return int
	 */
	abstract function num_rows();

	/**
	 * Returns the name of the specified field number in the result set.
	 * @param int $num the field number
	 * @return string
	 */
	abstract function field_name($num);

	/**
	 * Returns the type of the specified field number in the result set.
	 * @param int $num the field number
	 * @return string
	 */
	abstract function field_type($num);

	/**
	 * Fetches a record as an associative array from a result set.
	 *
	 * The heart of the result class, this function returns
	 * an associative array representing one row of the result set.
	 * Callers can specify whether they want numerically indexed
	 * or associative arrays, or both (the default).
	 * @param int $row the number of the row in the result set. can be
	 *                 null to fetch the next row in the set.
	 * @param int $method determines the kind of array to return
	 * @return array
	 */
	//XXX: FETCH_BOTH is noticeably slower for large queries; maybe we
	//     should consider changing the default
	abstract function fetch_array($row = FETCH_NEXTROW,
						$method = FETCH_BOTH);

	/**
	 * Frees the memory used by the query result.
	 *
	 * Note that further operations on this instance will be
	 * undefined! Returns true if the operation succeeded.
	 * @return bool
	 */
	protected function free_result()
	{
		unset($this->conn);
		unset($this->query);
	}

	/**
	 * Returns the number of affected rows of the query.
	 *
	 * Note: this works only for INSERT, UPDATE and DELETE queries.
	 * Use {@link num_rows} to determine the number of rows in
	 * a SELECT query.
	 * @return int
	 */
	function affected_rows()
	{
		return (isset($this->aff) ? $this->aff : -1);
	}

	/**
	 * Returns the last inserted id for the specified column and table.
	 * @param string $table the name of the table
	 * @param string $key the name of the primary key column
	 * @return int
	 */
	abstract function last_insert_id($table, $key);
}

/**
 * Class representing a connection to a database.
 *
 * Not intended to be instantiated by end users; see {@link db_get_object}
 * for documentation about connecting to a database.
 * @package sws
 */
abstract class db
{
	private $query_counter;
	protected $ign;

	/**
	 * Constructor for the class. This class is meant to be instantiated
	 * by {@link db_get_object}, not by end users.
	 * @access private
	 */
	function __construct()
	{
		$this->query_counter = 0;
		$this->ign = false;
	}

	/**
	 * Set whether errors are fatal.
	 *
	 * If this method is called with a argument that evaluate to true,
	 * all further database queries will complete silently, and errors
	 * will be ignored. It returns the previous value of the
	 * ignore_errors setting.
	 * @param bool $ign
	 * @return bool
	 */
	function ignore_errors($ign)
	{
		$i = $this->ign;
		$this->ign = $ign;

		return $i;
	}

	/**
	 * Connect to a database.
	 *
	 * This function connects to the given database. Not meant to be
	 * called by anyone else but {@link db_get_object}. Returns one
	 * of the database status codes.
	 * @param string $host the hostname or IP of the database server
	 * @param string $port the port on which the database server listens
	 * @param string $user the username to connect as
	 * @param string $passwd the password corresponding to the username
	 * @param string $dbname the name of the database to connect to
	 * @return int
	 * @see DB_OK
	 * @see DB_NOT_CONNECTED
	 * @see DB_NOT_SUPPORTED
	 * @access private
	 */
	abstract function connect($host, $user, $passwd, $dbname, $port = null);

	/**
	 * Perform an SQL query on the database.
	 *
	 * Perfoms the supplied query on the connected database, and returns
	 * a newly created {@link db_result} object (of the correct type) that
	 * represents the result.
	 * @param string $query the SQL query
	 * @return db_result
	 * @see db_result
	 */
	protected function query($query) {
		$this->query_counter++;
	}

	/**
	 * Begin a database transaction.
	 *
	 * Returns false if transactions are not supported.
	 * @return bool
	 */
	abstract function begin_transaction();

	/**
	 * Commit to a database transaction.
	 *
	 * Returns false if transactions are not supported.
	 * @return bool
	 */
	abstract function commit();

	/**
	 * Rollback a database transaction.
	 *
	 * Returns false if transactions are not supported.
	 * @return int
	 */
	abstract function rollback();

	/**
	 * Returns the SQL escaped version of the string.
	 *
	 * This uses the appropriate internal PHP function to escape the
	 * string in a database specific way, thus making it safe to use in
	 * queries. Always use this before sending user input to
	 * the database! It returns the escaped string surrounded by
	 * single quotes.
	 * @param string $str
	 * @return string
	 */
	abstract function quote($str);

	/**
	 * Compatability version of {@link quote}, to allow for older
	 * SWS code.
	 * @param string $str
	 * @deprecated 0.4.0
	 * @access private
	 */
	function escape_string($str)
	{
		return $this->quote($str);
	}

	/**
	 * Closes the database connection.
	 *
	 * While this should automatically be done by PHP, in case a script
	 * makes lots of connections this can be necessary to avoid
	 * resource shortages. Returns true on success or false on failure.
	 * @return bool
	 */
	abstract function close();

	/**
	 * Returns the amount of queries performed using this connection.
	 *
	 * Note that this only counts the number of queries executed using
	 * this object -- using multiple database objects will skew the
	 * results.
	 * @return int
	 */
	function query_count()
	{
		return $this->query_counter;
	}
}

?>
