<?php

/**
 * SWS database classes for PostgreSQL databases
 *
 * $Id: db_postgresql.php 570 2006-01-03 09:07:50Z stijn $
 * @package sws
 */

/**
 * Class representing a query result for a PostgreSQL database.
 * @package sws
 */
class db_postgresql_result extends db_result
{
	/**
	 * Initializes a PostgreSQL result class.
	 *
	 * Constructor for the class. This class is only meant to be
	 * instantiated by {@link db_postgresql::query}.
	 * @access private
	 */
	function __construct($conn, $query, $ign)
	{
		parent::__construct($conn, $query, $ign);

		$this->result = @pg_query($this->conn, $this->query);

		// Some rudimentary error checking
		if ($this->result === FALSE && !$this->ign)  {
			if (debugging_on()) {
				err_backtrace(
				    "Invalid query: " . $this->query . "\n" .
				    "Error: " . pg_last_error($this->conn));
			} else {
				// XXX: figure out a good error mechanism
				print "A database error occurred.\n";
			}
			exit;
		} else {
			$err = pg_last_error($this->conn);
			if ($err && debugging_on()) {
				err_backtrace(
				    "Query: " . $this->query . "\n" .
				    "Warning: " . $err . "\n");
			}
		}

		$this->aff = @pg_affected_rows($this->result);
	}

	function num_fields()  {
		return pg_num_fields($this->result);
	}

	function num_rows()  {
		return pg_num_rows($this->result);
	}

	function field_name($num)  {
		return pg_field_name($this->result, $num);
	}

	function field_type($num)  {
		return pg_field_type($this->result, $num);
	}

	function fetch_array($row = FETCH_NEXTROW, $method = FETCH_BOTH)  {
		if ($row == FETCH_NEXTROW)  {
			return pg_fetch_array($this->result, NULL, $method);
		}

		return pg_fetch_array($this->result, $row, $method);
	}

	function free_result()
	{
		parent::free_result();
		return pg_free_result($this->result);
	}

	function last_insert_id($table, $key)
	{
		if (!$this->conn) {
			return -1;
		}

		$res = pg_query($this->conn, "select currval('" . $table . "_" .
		    $key . "_seq')");
		if (!$res) {
			return -1;
		}

		$row = pg_fetch_array($res);
		return $row[0];
	}
}

/**
 * Class representing a connection to a PostgreSQL database.
 *
 * Not intended to be instantiated by end users; see {@link db_get_object}
 * for documentation about connecting to a database.
 * @package sws
 */
class db_postgresql extends db
{
	function connect($host, $user, $passwd, $dbname, $port = null)
	{
		if (!isset($this->conn) || $this->conn == 0) {
			if (!function_exists("pg_connect")) {
				return DB_NOT_SUPPORTED;
			}

			$this->conn = @pg_connect("host=$host " .
			    (!is_null($port) ? "port=$port " : "") .
			    "user=$user " .
			    ($passwd != "" ? "password=$passwd " : "") .
			    "dbname=$dbname");
			if (!$this->conn) {
				return DB_NOT_CONNECTED;
			}

			return DB_OK;
		}
	}

	function query($query)
	{
		parent::query($query);
		return new db_postgresql_result($this->conn, $query,
		    $this->ign);
	}

	function begin_transaction()
	{
		$res = pg_query($this->conn, "BEGIN TRANSACTION");
		if (!$res) {
			return false;
		}

		return true;
	}

	function commit()
	{
		$res = pg_query($this->conn, "COMMIT");
		if (!$res) {
			return false;
		}

		return true;
	}

	function rollback()
	{
		$res = pg_query($this->conn, "ROLLBACK");
		if (!$res) {
			return false;
		}

		return true;
	}

	function quote($str)
	{
		return "'" . pg_escape_string($str) . "'";
	}

	function close()
	{
		return pg_close($this->conn);
	}
}

?>
