<?php

/**
 * Login related definitions and functions.
 *
 * $Id$
 * @package sws
 */

/**#@+
 * <p>Value returned by the {@link verifyTicket} function when
 */
/**
 * Ticket signature verification error code.
 *
 * there was an error verifying the ticket signature.</p>
 */
define("TKT_ERROR", 1);
/**
 * Invalid ticket signature error code.
 *
 * the ticket signature is invalid.</p>
 */
define("TKT_FALSE", 2);
/**
 * Expired ticket error code.
 *
 * the ticket signature is valid but it has expired.</p>
 */
define("TKT_EXPIRED", 3);
/**
 * No ticket status code
 *
 * the user did not present a ticket with this request.</p>
 */
define("TKT_NOTFOUND", 4);
/**#@-*/

/**
 * Login required status code.
 *
 * Used by the login code to indicate that authentication is required.
 */
define("LOGIN_REQUIRED", 1); 
/**#@+
 * <p>Used by the login code to indicate an error that requires reauthentication
 * because
 */
/**
 * Failed login status code.
 *
 * the previous login attempt failed.</p>
 */
define("LOGIN_FAILED", 2); 
/**
 * Not enough privileges status code.
 *
 * the previous login attempt succeeded but the autenticated user does not
 * have enough privileges.</p>
 */
define("LOGIN_PRIVS", 3); 
/**
 * Authentication expired status code.
 *
 * the authentication cookie has expired.</p>
 */
define("LOGIN_EXPIRED", 4); 
/**
 * Login configuration error status code.
 *
 * of an error in the login configuration. Most of the time this will fail
 * continually until the administrator intervenes, but at least
 * with this status the theme can present a reason for failing logins.</p>
 */
define("LOGIN_CONFIG", 5); 
/**#@-*/

/**
 * Change password status code.
 *
 * Used by the password change code to indicate the initiation of a
 * password change.
 */
define("PASSWORD_CHANGE", 1); 
/**#@+
 * <p>Used by the password change code to indicate an error that requires
 * re-entering of the password information because
 */
/**
 * Wrong password status code.
 *
 * the password entered was not correct for the user changing the password.</p>
 */
define("PASSWORD_WRONG", 2); 
/**
 * Password mismatch status code.
 *
 * the newly chosen password fields did not match.</p>
 */
define("PASSWORD_MISMATCH", 3); 
/**#@-*/

/**
 * Verifies and returns the user's decrypted ticket cookie.
 *
 * This function is the heart of the user authentication system.
 * It tries to verify the user's ticket cookie by decoding it,
 * verifying it cryptographically, and then testing whether it
 * has expired.
 * @return mixed Either an error code indicating an invalid ticket or the ticket
 * @see TKT_FALSE
 * @see TKT_ERROR
 * @see TKT_EXPIRED
 * @see TKT_NOTFOUND
 */
function verifyTicket()
{
	if (!isset($_COOKIE['tkt'])) {
		return TKT_NOTFOUND;
	}

	if (strchr($_COOKIE['tkt'], "-") === FALSE) {
		return TKT_FALSE;
	}

	list($tkt, $tktsign) = explode("-", $_COOKIE['tkt']);
	$tkt = base64_decode($tkt);
	$tktsign = base64_decode($tktsign);

	$config = configPath();
	$pubid = openssl_get_publickey("file://" . swsroot() .
	    "/etc/keys/sws.crt");
	if ($pubid === FALSE) {
		// public key file not found or unreadable
		return TKT_ERROR;
	}

	$res = openssl_verify($tkt, $tktsign, $pubid);
	switch ($res) {
	case 1:
		break;
	case 0:
		return TKT_FALSE;
	default:
		return TKT_ERROR;
	}

	$tkt = unserialize($tkt);

	if ($tkt['useragent'] != md5($_SERVER['HTTP_USER_AGENT'])) {
		return TKT_FALSE;
	}

	if ($tkt['ts'] > time() || $tkt['ts'] < time() - SESSION_LIFETIME) {
		return TKT_EXPIRED;
	}

	return $tkt;
}

/**
 * Returns true if a user is succesfully authenticated.
 * @return bool true if user is logged in, false if not
 */
function loggedIn()
{
	return is_array(verifyTicket());
}

/**
 * Determine whether privilege $priv is in the set $set.
 * @param string $priv privilege to check for
 * @param array $set array of privileges to search in
 * @return bool true if the privilege is in the set, false otherwise
 */
function privInSet($priv, $set)
{
	if (!is_array($priv)) {
		$privs = array($priv);
	} else {
		$privs = $priv;
	}

	foreach ($privs as $p) {
		if (in_array($p, $set)) {
			return true;
		}
	}

	return false;
}

/**
 * List privileges for a user.
 *
 * Returns an array of privilege names that the requested user has. If no
 * user was specified, it returns the privileges for the current user. If
 * the specified user id was not found, it returns -1.
 * @param int $userid the ID of the user to list privileges for (default: current user)
 * @return mixed array of privilege names assigned to the user or -1 if user not found
 */
function listPrivs($userid = -1)
{
	if ($userid == -1) {
		$userid = userId();
		if ($userid == -1) {
			return -1;
		}
	}

	$db = db_get_object("sws");

	$res = $db->query(
	    "SELECT privs.name AS priv_name " .
	    "FROM privs, user_privs " .
	    "WHERE user_privs.user_id = " . $userid . " " .
		"AND user_privs.priv_id = privs.id");
	$privs = array();
	while ($row = $res->fetch_array()) {
		$privs[] = $row['priv_name'];
	}

	return $privs;
}

/**
 * Determine whether the current user has a certain privilege (out of a set).
 *
 * This returns true if the current user was granted the specified privilege,
 * or one out of the set of specified privileges.
 * @param mixed $priv the privilege or set of privileges to check for
 * @return bool true if the current user has been granted (one of) the privilege(s)
 */
function hasPriv($priv = "")
{
	$tkt = verifyTicket();
	if (!is_array($tkt)) {
		return false;
	}

	if (!$priv || $priv == "" || (is_array($priv) && sizeof($priv) == 0)) {
		return true;
	}

	return privInSet($priv, listPrivs($tkt['id']))
		|| privInSet("admin", listPrivs($tkt['id']));
}

/**
 * Starts a login session.
 *
 * Note that the page must not output anything before calling this, because
 * of the location redirect.
 *
 * This function does not return.
 * @access private
 * @param mixed $privs one or more privs to check for (string, array of strings)
 * @param int $reason login status code for this login
 * @see LOGIN_REQUIRED
 * @see LOGIN_FAILED
 * @see LOGIN_PRIVS
 * @see LOGIN_EXPIRED
 * @see LOGIN_CONFIG
 */
function startLogin($privs, $reason = LOGIN_REQUIRED)
{
	saveState("login");
	redirectLogin("login.php" .
	    "?privs=" . (is_array($privs) ? urlencode(implode(",", $privs)) :
		    urlencode($privs)) .
 	    "&back=" . urlencode(completeQualifiedMe()) .
	    ($reason ? "&reason=" . $reason : ""));
	exit;
}

/**
 * Requires an authenticated user before continuing with the page.
 *
 * This function must be called before anything is output to the browser.
 * It requires that a valid user has authenticated, and optionally requires
 * specific privilege(s). This function does not return unless these
 * requirements are met (and will in fact redirect the user to another
 * page to present his credentials if those requirements are not met).
 * @param mixed $privs a string or an array of strings of privileges to check against
 */
function requireLogin($privs = "")
{
	if (isset($_COOKIE['swsdone']) && $_COOKIE['swsdone'] == "login") {
		unsetLoginCookie('swsdone');
	}

	$tkt = verifyTicket();
	switch ($tkt) {
	case TKT_NOTFOUND:
		startLogin($privs);
	case TKT_FALSE:
		startLogin($privs, LOGIN_FAILED);
	case TKT_ERROR:
		startLogin($privs, LOGIN_CONFIG);
	case TKT_EXPIRED:
		startLogin($privs, LOGIN_EXPIRED);
	default:
		if (!is_array($tkt)) {
			//XXX: need proper error mechanism
			print "An unknown error with your ticket was detected.";
			exit;
		}
	}

	if (!hasPriv($privs)) {
		startLogin($privs, LOGIN_PRIVS);
	}
}

/**
 * Allows users to change their password before continuing the page.
 *
 * This function will not return unless the change password process
 * has been completed (and will in fact redirect the user to the
 * change password page in the meantime).
 */
function changePassword()
{
	if (isset($_COOKIE['swsdone']) && $_COOKIE['swsdone'] == "changepw") {
		unsetLoginCookie('swsdone');
		return;
	}

	if (!loggedIn()) {
		requireLogin();
	}

	saveState("changepw");
	redirectLogin("password.php?back=" . urlencode(completeQualifiedMe()));
	exit;
}

/**
 * Logs out the current user before continuing the page.
 *
 * This function will not return unless the logout process has been
 * completed.
 */
function logout()
{
	if (isset($_COOKIE['swsdone']) && $_COOKIE['swsdone'] == "logout") {
		unsetLoginCookie('swsdone');
		return;
	}

	if (!loggedIn()) {
		return;
	}

	saveState("logout");
	redirectLogin("logout.php?back=" . urlencode(completeQualifiedMe()));
	exit;
}

/**
 * Return true if the current request was for the login host.
 */
function onLoginHost()
{
	$config = configPath();
	$loginHost = $_SESSION["conf-$config"]['SWS']['loginhost'];

	if ($loginHost == "" || $_SERVER['HTTP_HOST'] == $loginHost) {
		return true;
	}

	return false;
}

/**
 * Construct the sws login related URL.
 * @param string $url the url to append (without $swsurl/login)
 * @access public
 */
function makeLoginURL($url = "")
{
	$config = configPath();

	// check if there is a login host configured and set the host
	// part of the URL
	$loginHost = $_SESSION["conf-$config"]['SWS']['loginhost'];

	if ($loginHost == "") {
		$host = addSlash($_SERVER['HTTP_HOST']);
	} else {
		$host = addSlash($loginHost);
	}
	
	$loginURL =
	    addSlash(stripSlash($_SESSION["conf-$config"]["SWS"]['swsurl'])) . "login/" ;
	
	// check if we are using https or http and set the protocol part
	// of the URL
	$proto = $_SESSION["conf-$config"]['sws']['https'] ? "https://" : "http://" ;

	return $proto . $host . $loginURL . $url ;
}

/**
 * Redirect to a login related page.
 * @param string $url the url to redirect to (without $swsurl/login)
 * @access private
 */
function redirectLogin($url = "")
{
	header("Location: " .
		makeLoginURL($url));
}

/**
 * Sets a cookie to be used in the login process.
 *
 * Note that these are always session cookies (ie discarded when the
 * useragent is closed).
 * @param string $name the name of the cookie
 * @param string $contents the contents of the cookie
 * @see unsetLoginCookie
 * @access private
 */
function setLoginCookie($name, $contents)
{
	$config = configPath();
	setcookie($name, $contents, 0, '/',
	    $_SESSION["conf-$config"]['SWS']['logindomain']);
	$_COOKIE[$name] = $contents;
}

/**
 * Unset a login cookie.
 *
 * Forcibly resets a cookie from being returned by user agents for
 * new requests.
 * @param string name the name of the cookie to discard.
 * @access private
 */
function unsetLoginCookie($name)
{
	$config = configPath();
	setcookie($name, '', time() - 42000, '/',
	    $_SESSION["conf-$config"]['SWS']['logindomain']);
	unset($_COOKIE[$name]);
}

/**
 * Log a failed login to the database.
 * @param object $db the database connection handle
 * @param string $user the username
 * @param int $reason login failure reason
 * @access private
 */
function logFailedLogin($db, $user, $reason = LOGIN_FAILED)
{
        $db->ignore_errors(1);
        $db->query("INSERT INTO logins (username, ip, failure) VALUES (" .
                $db->quote($user) . ", " . $db->quote(remoteAddress()) . ", " .
                $reason . ")");
        $db->ignore_errors(0);
}

/**
 * Log the logout time of a session.
 *
 * It doesn't really matter if we don't find the record and therefore
 * nothing gets updated -- only the log will be wrong.
 * @param array $tkt the login ticket of the session
 * @access private
 */
function logLogoutTime($tkt)
{
	if (!isset($tkt['start'])) {
		return;
	}

	$db = db_get_sws_privileged_object();
	$db->ignore_errors(1);
	$db->query("UPDATE wtmp SET s_end = NOW(), start = start " .
	    "WHERE user_id = " . $tkt['id'] . " " .
		"AND ip = " . $db->quote($tkt['remote_addr']) . " " .
		"AND start = " . $db->quote($tkt['start']));
	$db->ignore_errors(0);
}

/**
 * Default redirect url for login procedures, for when a user explicitly
 * requests a login or logout page and so we don't know where to go at the
 * end of the request.
 * @return string $url the url to redirect to
 */
function defaultBackURL()
{
	$config = configPath();
	//XXX actually we can do better, examine REFERER and check it
	//    against known vhosts, if a match found we use that.
	$host = $_SESSION["conf-$config"]['SWS']['vhosts']['_default_'];
	$site = $host['sites'][0];
	return "http://" . $host['hostname'] . "/" .
	    stripSlash($host[$site]['baseurl']);
}

?>
