<?php

/**
 * Miscellaneous functions that do not really fit elsewhere.
 *
 * $Id: misc.php 499 2005-08-19 08:41:14Z stijn $
 * @package sws
 */

/**
 * Site configuration disallows https status value.
 */
define("NOT_CONFIGGED_FOR_HTTPS", -1);

/**
 * Strip the query string of an (optional) GET request URL, and return it.
 * @param string $url the URL
 * @return string the URL, with the GET request portion (?foo=bar&xyzzy=frob)
 *                stripped
 */
function stripQuerystring($url)
{
	if ($querypos = strpos($url, '?')) {
		return substr($url, 0, $querypos);
	} else {
		return $url;
	}
}

/**
 * Strip the PATH_INFO of an URL.
 * @param string $url the URL
 * @return string the URL with the optional PATH_INFO information stripped
 */
function stripPathInfo($url)
{
	if (!isset($_SERVER['PATH_INFO'])) {
		return $url;
	}

	if ($querypos = strpos($url, $_SERVER['PATH_INFO'])) {
		return substr($url, 0, $querypos);
	} else {
		return $url;
	}
}

/**
 * Return the canonicalized referrer of the current webpage.
 *
 * Canonicalized because the query string of the referrer is stripped before
 * being returned. Returns "" if no referrer information was present in the
 * request.
 * @return string the referrer of the webpage or ""
 */
function getReferrer()
{
	return isset($_SERVER['HTTP_REFERER']) ?
	    stripQuerystring($_SERVER['HTTP_REFERER']) : "";
}

/**
 * Return the complete URL to the current page, including the query string
 * but without the 'http://' part.
 * @return string the complete URL to the current page
 */
function completeMe()
{
    $my_url = isset($_SERVER['REQUEST_URI']) ?
        $_SERVER['REQUEST_URI'] : $_SERVER['PHP_SELF'] . 
		(isset($_SERVER['QUERY_STRING']) ? 
			'?' . $_SERVER['QUERY_STRING'] : '');
    return $my_url;
}

/**
 * Return the complete URL to the current page, without the query string
 * (if any) and without the 'http://' part.
 * @return string the complete URL to the current page without the query string
 */
function me()
{
	return stripQuerystring(completeMe());
}

/**
 * Return the complete URL to the current page, including the query string
 * and the 'http://' or 'https://' part.
 * @return string the complete URL to the current page including prefix
 */
function completeQualifiedMe()
{
    $protocol = (isset($_SERVER['HTTPS']) &&
        $_SERVER['HTTPS'] == "on") ? "https://" : "http://";
    $url_prefix = $protocol . $_SERVER['HTTP_HOST'];
    return $url_prefix . completeMe();
}

/**
 * Return the complete URL to the current page, without the query string
 * but with the 'http://' or 'https://' part.
 * @return string the complete URL to the current page including prefix,
 *                but without the query string.
 */
function qualifiedMe()
{
    return stripQuerystring(completeQualifiedMe());
}

/**
 * Return the complete URL to the current page, including the query string,
 * prefixed with 'https://'.
 * @return string the complete URL to the current page, including the query
 *                string, prefixed with 'https://'.
 */
function completeEncryptedMe()
{
    return "https://" . $_SERVER['HTTP_HOST'] . completeMe();
}

/**
 * Return the complete URL to the current page, without the query string,
 * prefixed with 'https://'.
 * @return string the complete URL to the current page, without the query
 *                string, prefixed with 'https://'.
 */
function encryptedMe()
{
	return stripQuerystring(completeEncryptedMe());
}

/**
 * Return true if the referrer to the current page matches.
 *
 * The function defaults to checking whether the referrer is the same
 * as the current page, either in SSL or non-SSL mode. An optional
 * referrer may be specified to match against.
 * @param string $goodReferrer the referrer to match against
 * @return bool true if a referrer was specified and it matched
 */
function matchReferrer($goodReferrer = "")
{
	if ($goodReferrer == "") {
		$unencRef = "http://" . $_SERVER['HTTP_HOST'] . me();
		$encRef   = encryptedMe();
		return ($unencRef == getReferrer() ||
		    $encRef == getReferrer());
	}

	return ($goodReferrer == getReferrer());
}

/**
 * Return a username if the current request is for a user site.
 *
 * Returns "" if the current request is not for a user site. 
 * @return string the username of the user for the current site request
 */
function getUserUser()
{
	preg_match('/\/(~|%7(E|e))([^\/]*)\//', me(), $user);
	if (isset($user[3]) && $user[3] != "") {
		return $user[3];
	} else {
		return "";
	}
}

/**
 * Return the base URL for the user site if the current request is for a user
 * site.
 *
 * Returns "" if the current request is not for a user site.
 * @return string the base URL for the user site.
 */
function getUserURL()
{
	preg_match('/\/(~|%7(E|e))([^\/]*)\//', me(), $user);
	if (isset($user[0]) && $user[0] != "") {
		return $user[0];
	} else {
		return "";
	}
}

/**
 * Redirect the user to the encrypted version of the current URL.
 *
 * Will not work if output has been sent to the browser before calling.
 * Respects the configuration setting for 'https' enabled sites.
 * Returns only if the page is requested by secure a method.
 */
function encryptProtocol()
{
	// Only use HTTPS if SWS is configured for it
	$config = configPath();
	if ($_SESSION["conf-$config"]['sws']['https'] == 0)  {
		return NOT_CONFIGGED_FOR_HTTPS;
	}

	$protocol = (isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] == 'on') ?
	    "https://" : "http://";

	$loc = "https://" . $_SERVER['SERVER_NAME'] . me();

	// Redirect browser to encrypted page
	if ($protocol != "https://") {
		if ($_SERVER['QUERY_STRING']) {
			header("Location: " . $loc . "?" .
			    $_SERVER['QUERY_STRING']);
		} else {
			header("Location: " . $loc);
		}
		exit;
	}
}

/**
 * Redirect the user to the unencrypted version of the current URL.
 *
 * Basically the opposite of {@link encryptProtocol}, this function
 * works about the same. No output may have been sent to the browser yet,
 * and it will only return if the page is requested using an insecure
 * method.
 */
function unencryptProtocol()
{
	$protocol = (isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] == 'on') ?
	    "https://" : "http://";

	$loc = 'http://' . $_SERVER['SERVER_NAME'] . completeMe();

	// Redirect browser to unencrypted page
	if ($protocol != "http://") {
		// Try to prevent browsers from resending secure post data
		header('HTTP/1.1 303 See Other');

		if ($_SERVER['QUERY_STRING']) {
			header("Location: " . $loc . "?" .
			    $_SERVER['QUERY_STRING']);
		} else {
			header("Location: " . $loc);
		}
		exit;
	}
}

/**
 * Returns true if insecure logins are allowed.
 *
 * Insecure logins are logins using the unencrypted http:// method.
 * @return bool true if insecure logins are allowed
 */
function allowInsecureLogin()
{
	$config = configPath();
	return (isset($_SESSION["conf-$config"]['sws']['allow_insecure_login'])
	    && $_SESSION["conf-$config"]['sws']['allow_insecure_login'] ?
	    true : false);
}

/**
 * Return the address of the client, as specified by the client.
 *
 * <b>DO NOT USE FOR SECURITY PURPOSES!</b>. This is not to be trusted.
 * Any client can fake this. It is only useful for identifying the
 * same client during web requests.
 *
 * Returns "unknown" if the remote client address was not sent along with
 * the request.
 * @return string the remote address or "unknown"
 */
function remoteAddress()
{
	if (isset($_SERVER["HTTP_X_FORWARDED_FOR"])) {
		return $_SERVER["HTTP_X_FORWARDED_FOR"];
	}

	if (isset($_SERVER["REMOTE_ADDR"])) {
		return $_SERVER["REMOTE_ADDR"];
	}

	return "unknown";
}

?>
