<?php

/**
 * Session management functions.
 *
 * This part of the framework does not include user callable functions.
 * It is purely intended to allow SWS more control over sessions, by
 * defining it's own session handlers.
 *
 * $Id: session.php 572 2006-01-03 09:13:07Z stijn $
 * @package sws
 */

/**
 * The lifetime of a session, in seconds.
 */
// XXX: this should be gettable from php.ini, as session.gc_maxlifetime
define('SESSION_LIFETIME', 60 * 60 * 2);

/**
 * Open session callback handler.
 *
 * This simply connects to the framework database, to make sure we can
 * save sessions.
 * @param string $arg session identifier
 * @return bool true if the session store could be opened
 * @access private
 */
function sess_open($arg)
{
	db_get_object("sws");
	return true;
}

/**
 * Close session callback handler.
 *
 * Does nothing.
 * @return bool true if the session store could be closed
 * @access private
 */
function sess_close()
{
	return true;
}

/**
 * Read a specific session callback.
 * @param string $key the session ID
 * @return string the value of the session if it could be found in the
 *                session store, or "" otherwise.
 * @access private
 */
function sess_read($key)
{
	$db = db_get_object("sws");
	$res = $db->query("SELECT value FROM sessions WHERE sesskey = " .
	    $db->quote($key) . " AND expiry > " . time());
	if (!$res || $res->num_rows() == 0) {
		return "";
	}

	if (list($val) = $res->fetch_array()) {
		return $val;
	}

	return "";
}

/**
 * Write session callback
 *
 * This function also deletes old sessions, 
 * @param string $key the session ID
 * @param string $val the value of the session
 * @return bool true if the session was succesfully stored
 * @access private
 */
/*
 * As an insightful comment on php.net notes, there is a 'race' present in
 * the session read/write code:
 *
 *	script "A" start	.
 *	read session data	.
 *	.			script "B" start
 *	.			read session data
 *	running (30secs)	add session data
 *	.			write session data
 *	.			script "B" stop
 *	write session data	.
 *	script "A" stop		.
 *
 * So, B's modified session data is overwritten by A's.
 *
 * The only way to prevent this is to enforce ordering on writes, but that
 * would imply that parallel HTTP access is serialized. Clearly this is not
 * desirable.
 *
 * Therefore, the 'solution' is left up to long running script; those scripts
 * should reread the session data just before the end of the page, if they
 * need to modify the session data in any way.
 *
 * Note that this does not really fix the race, it only decreases the interval
 * window by a large amount. Tests should be done to see what the actual
 * performance decrease is for concurrent access if the serialize option is
 * implemented. If that is acceptable, the ordering should be enforced to
 * really close it.
 */
function sess_write($key, $val)
{
	$time = time();
	$expiry = $time + SESSION_LIFETIME;

	# First delete old sessions, otherwise nothing will get updated.
	$db = db_get_object("sws");
	$res = $db->query("DELETE FROM sessions WHERE expiry <= " . $time);
	$res = $db->query("SELECT sesskey FROM sessions WHERE sesskey = " .
	    $db->quote($key));
	if ($res->num_rows() == 0) {
		$res = $db->query("INSERT INTO sessions (sesskey, expiry, " .
		    "value) VALUES (" . $db->quote($key) .
		    ", $expiry, " . $db->quote($val) . ")");
	} else {
		$res = $db->query("UPDATE sessions SET expiry = $expiry, " .
		    "value = " . $db->quote($val) . " WHERE " .
		    "sesskey = " . $db->quote($key) . " AND " .
		    "expiry > " . $time);
	}

	return true;
}

/**
 * Destroy session callback
 * @param string $key the session ID for the session that must be destroyed
 * @return bool true if the session could be destroyed
 * @access private
 */
function sess_destroy($key)
{
	$db = db_get_object("sws");
	return ($db->query("DELETE FROM sessions WHERE sesskey = " .
	    $db->quote($key)) ? true : false);
}

/**
 * Return the decoded value of user settings from a session value
 *
 * Needed for recording session expired logouts, used by {@link sess_gc}.
 * @param string $value the session value to extract from
 * @return array the user hash for the current user, or "" if no user
 *               was logged in in this session
 * @access private
 */
function sess_userconfig($value)
{
	$config = configPath();
	$s = preg_replace('#^.*user-' . $config . '\|([^|]*).*$#',
		'$1', $value);
	if ($s == $value) {
		return "";
	}

	$sess = unserialize($s);
	return $sess;
}

/**
 * Session garbage collect callback
 *
 * Called by PHP periodically.
 * @param int $maxlifetime the max lifetime of the session according to PHP
 *                         No idea in what unit or how this gets determined,
 *                         and the PHP manual is silent about it too.
 * @return bool true if the garbage collect operation succeeded
 * @access private
 */
function sess_gc($maxlifetime)
{
	$db = db_get_object("sws");
	$db->ignore_errors(1);

	$time = time();

	$res = $db->query("SELECT value FROM sessions " .
	    "WHERE expiry <= " . $time);
	while ($res && $row = $res->fetch_array()) {
		$user = sess_userconfig($row["value"]);
		if (sizeof($user) != 0 && isset($user["id"])
		    && isset($user["remote_addr"]) && isset($user["start"])) {
			$db->query("UPDATE wtmp SET s_end = NOW(), " .
			    "start = start WHERE user_id = " . $user["id"] .
			    " AND ip = " . $db->quote($user["remote_addr"]) .
			    " AND start = " . $db->quote($user["start"]));
		}
	}

	return ($db->query("DELETE FROM sessions WHERE expiry <= " . $time) ?
	    true : false);
}

/**
 * Restarts the current session.
 *
 * Also generates a new session ID to prevent session fixation attacks,
 * and rereads the SWS configuration in the new session.
 */
function sess_restart()
{
	$_SESSION = array();
	if (isset($_COOKIE[session_name()])) {
		setcookie(session_name(), '', time() - 42000, '/');
	}
	session_destroy();

	// apparently, PHP forgets the save handlers for destroyed sessions
	session_set_save_handler("sess_open", "sess_close", "sess_read",
	    "sess_write", "sess_destroy", "sess_gc");
	session_start();

	session_regenerate_id();

	setConfig();
}

session_set_save_handler("sess_open", "sess_close", "sess_read",
    "sess_write", "sess_destroy", "sess_gc");

?>
