<?php

/**
 * Site URL manipulation functions.
 *
 * $Id: sitepath.php 562 2005-12-02 13:41:38Z stijn $
 * @package sws
 */

/**
 * Strip double slashes from an URL.
 *
 * Although Apache 2.0.* doesn't seem to allow them in URLs, Apache 1.3.* does,
 * and we don't want them in the database. Other environments (IIS?) might
 * allow them too, but that is untested.
 * @param string $str the URL to strip double slashes from
 * @return string the URL without '//' in it
 */
function stripDoubleSlashes($str)
{
	return implode("/", preg_split("/\//", $str, -1, PREG_SPLIT_NO_EMPTY));
}

/**
 * Normalize the given URL to an URL used in the database.
 *
 * In practice, this means adding 'index.php' to directories, and stripping
 * double slashes from the URL.
 * @param string $url the URL to normalize
 * @return string the normalized URL
 */
function normalizeURL($url)
{
	$dir = ($url[strlen($url) - 1] == "/" ? true : false);
	$root = ($url == "/");
	$url = ($root ? "" : "/") . stripDoubleSlashes($url) .
	    ($dir ? "/index.php" : "");

	return $url;
}

/**
 * Return the title for an URL.
 *
 * Returns the last element of the url (minus / or .php) if the URL was
 * not found in the database.
 * @param string $url the URL to get the title for
 * @return string the title for the URL
 */
function getURLTitle($url)
{
	$url = normalizeURL($url);

	$db = db_get_object("sws");
	$config = configPath();
        $res = $db->query("SELECT * FROM pages " .
	    "WHERE hostname = " .
		$db->quote($_SESSION["conf-$config"]['sws']['hostname']) .
		" AND url = " . $db->quote($url));
	if ($res->num_rows() == 0) {
		// Strip last / or .php
		$name = str_replace("/index.php", "", $url);
		$name = str_replace(".php", "", $name);
		$name = substr(strrchr($name, "/"), 1);
		if (!$name) {
			$name = "index";
		}
		return $name;
	}

	$row = $res->fetch_array();
	$res->free_result();
	return $row['title'];
}

/**
 * Set or update the title of a given URL in the database.
 * @param string $url the URL to set the title for
 * @param string $title the title for the URL
 * @param bool $inindex whether this page should be listed in an index
 */
function setURLTitle($url, $title, $inindex)
{
	$url = normalizeURL($url);
	$db = db_get_object("sws");
	$config = configPath();
        $res = $db->query("SELECT * FROM pages " .
	    "WHERE hostname = " .
		$db->quote($_SESSION["conf-$config"]['sws']['hostname']) .
		" AND url = " . $db->quote($url));

	// Note that REPLACE cannot replace INSERT/UPDATE because it would
	// add an entry if any of title or inindex differs.
	if ($res->num_rows() == 0) {
		$db->query("INSERT INTO pages (hostname, url, " .
		    "title, inindex) VALUES (" .
		    $db->quote($_SESSION["conf-$config"]['sws']['hostname']) .
		    ", " . $db->quote($url) . ", " . $db->quote($title) .
		    ", " . ($inindex ? "1" : "0") . ")");
	} else {
		$db->query("UPDATE pages SET title = " .
		    $db->quote($title) . ", inindex = " .
		    ($inindex ? "1" : "0") . " WHERE hostname = " .
		    $db->quote($_SESSION["conf-$config"]['sws']['hostname']) .
		    " AND url = " . $db->quote($url));
	}
	$res->free_result();
}

/**
 * Return an array with breadcrumb info.
 *
 * This function walks the path to the current file and return an array
 * in the form of (href, title) pairs. This array is in order of the
 * tree (so getPathInfo("/bla/bla2") would return an array
 *
 * <pre>
 * (
 *     array("bla", "titel for bla"),
 *     array("bla2", "titel for bla2")
 * )
 * </pre>
 * @param string $url the URL for the page
 * @return array the array of breadcrumbs
 */
function getPathInfo($url)
{
	global $PAGE;

	$url = normalizeURL($url);
	$config = configPath();

	if (isset($PAGE['options']['error'])
	    || (strpos($url,
	   $_SESSION["conf-$config"]['SWS']['swsurl'] . "/login/") !== FALSE)) {
		return array(array($_SESSION["conf-$config"]['sws']['baseurl'] .
			"index.php",
		    getURLTitle($_SESSION["conf-$config"]['sws']['baseurl'] .
			"index.php")));
	}

	$path = array();
	$dirs = explode("/", dirname($url));
	$pathidx = 0;
	$running = $_SESSION["conf-$config"]['sws']['baseurl'];

	// Insert main page
	$path[$pathidx++] = array($running,
				getURLTitle($running . "index.php"));

	// Insert rest of path
	for ($i = sizeof(explode("/",
	    $_SESSION["conf-$config"]['sws']['baseurl'])) - 1;
	    $i < sizeof($dirs); $i++) {
		if ($dirs[$i] == "") {
			continue;
		}

		$running .= $dirs[$i] . "/";

		$path[$pathidx++] = array($running,
			getURLTitle($running . "index.php"));
	}

	if (!strstr($url, "/index.php")) {
		$path[$pathidx++] = array($url, getURLTitle($url));
	}

	return $path;
}

/**
 * Returns all topics based on the current URL.
 *
 * Topics are:
 *  - subdirectories with a file index.php in them
 *  - files that end in .php in the current directory
 *  - explicitly included files/directories by listing them in the 'include'
 *    page option, separated by comma's. External URLs can be included
 *    by using the form 'Title|http://some/url'.
 *
 * Topics are returned in an array, sorted by title in natural order. A page
 * can override this by specifying the 'topicorder' page option, also
 * separated by comma's.
 *
 * Topics can be excluded by being listed in the 'exclude' page option, again
 * separated by comma's if necessary.
 *
 * The array returned is in the same format as {@link getPathInfo}, each
 * element being a (href, title) tuple.
 * @param string $url the URL to generate the topics array for.
 * @return array the topics array.
 */
function getTopics($url)
{
	global $PAGE;

	$url = normalizeURL($url);
	$urlDir = (dirname($url) != "/" ? addSlash(dirname($url)) : "/");
	$directory = dirname($_SERVER['SCRIPT_FILENAME']) . "/";

	$rawtopics = array();

	// First determine the topics to exclude from page options
	$exclude = array();
	if (isset($PAGE['options']['exclude'])) {
		$exclude = explode(",", $PAGE['options']['exclude']);
	}

	// Next check the topics from the files in the current directory
	$dir = dir($directory);
	while ($entry = $dir->read()) {
		// Skip ., .., admin, index.php
		if ($entry == "." || $entry == ".." || $entry == "admin" ||
		    $entry == "index.php") {
			continue;
		}

		// Skip explicitly excluded entries
		if (in_array($entry, $exclude) ||
		    (is_dir($directory . $entry) &&
		     in_array($entry . "/", $exclude))) {
			continue;
		}

		// Skip regexp matched excluded entries
		foreach ($exclude as $ex) {
			if (preg_match("/" . $ex . "/", $entry) == 1) {
				continue 2;
			}
		}

		// Check for subdirectories that have an index.php
		if (is_dir($directory . $entry) &&
		    is_file($directory . $entry . "/index.php")) {
			$rawtopics[$entry] = array($urlDir . $entry .
			    "/index.php", getURLTitle($urlDir . $entry .
				"/index.php"));
		}

		// Check for *.php files
		if (is_file($directory . $entry) &&
		    strstr($entry, ".php") == ".php") {
			$rawtopics[$entry] = array($urlDir . $entry,
			    getURLTitle($urlDir . $entry));
		}

		// Skip the rest
	}
	$dir->close();

	// Last include the explicit topics from page options
	$include = array();
	if (isset($PAGE['options']['include'])) {
		$include = explode(",", $PAGE['options']['include']);

		foreach ($include as $entry) {
			// Check for subdirectories that have an index.php
			if (is_dir($directory . $entry) &&
			    is_file($directory . $entry . "/index.php")) {
				$rawtopics[$entry] =
				    array($urlDir . $entry . "/index.php",
					getURLTitle($urlDir . $entry .
					    "/index.php"));
				continue;
			}

			// Check for *.php files
			if (is_file($directory . $entry)) {
				$rawtopics[$entry] = array($urlDir . $entry,
				     getURLTitle($urlDir . $entry));
				continue;
			}
 			if (is_file($directory . $entry . ".php")) {
				$rawtopics[$entry] = array($urlDir . $entry .
				    ".php",
				     getURLTitle($urlDir . $entry . ".php"));
				continue;
			}

			// Else assume this is something else (e.g. an external
			// link); don't mess it up, but check to see if
			// there's a title provided (separate by a bar)
			$ent = explode("|", $entry);
			if (isset($ent[1])) {
				$rawtopics[$ent[1]] = array($ent[1], $ent[0]);
			} else {
				$rawtopics[$entry] = array($entry, $entry);
			}
		}
	}

	// Now create the final topics array
	$topics = array();

	// Sort first according to the topicsorder option if specified.
	if (isset($PAGE['options']['topicorder'])) {
		$order = explode(",", $PAGE['options']['topicorder']);

		foreach ($order as $entry) {
			if (isset($rawtopics[$entry])) {
				$topics[] = $rawtopics[$entry];
				unset($rawtopics[$entry]);
			} else if (isset($rawtopics[$entry . '.php'])) {
				$topics[] = $rawtopics[$entry . '.php'];
				unset($rawtopics[$entry . '.php']);
			}
		}
	}

	// Now sort the topics that weren't explicitly ordered
	foreach($rawtopics as $i => $val)  {
		$url = $rawtopics[$i][0];
		$title = $rawtopics[$i][1];
		$titles[$url] = $title;
	}

	if (isset($titles) && sizeof($titles) != 0) {
		natsort($titles);
		foreach($titles as $url => $title)  {
			$rwsrt[] = array($url, $title);
		}
		$rawtopics = $rwsrt;
	}

	// Merge the explicitly included topics and the sorted topics
	$topics = array_merge($topics, $rawtopics);

	return $topics;
}

/**
 * Converts an array of hrefs to a topics array.
 *
 * Currently only used for pages that specify an explicit list of topics
 * using the 'topics' page option.
 * @param array $topics a list of URLs to generate topics for
 * @return array a list of ($topic, $title) pairs
 * @access private
 */
function convertTopics($topics)
{
	$t = array();

	for($i = 0; $i < sizeof($topics); $i++) {
		$dir = dirname($_SERVER['SCRIPT_NAME']) . "/";
		if (substr($dir, -1) != "/") {
			$dir .= "/";
		}
		$title = getURLTitle($dir . $topics[$i]);
		if ($title == $topics[$i]) {
			$title = getURLTitle($dir . $topics[$i] . "/index.php");
		}
		$t[$i] = array($topics[$i], $title);
	}

	return $t;
}

/**
 * Compute the link to one directory level up for an URL
 * @param string $url the URL to calculate the backlink for
 * @return string the URL to the directory above the last
 */
function getBacklink($url)
{
	global $PAGE;

	$url = normalizeURL($url);

	$config = configPath();
	if ((addSlash(dirname($url)) == addSlash($_SESSION["conf-$config"]['sws']['baseurl']))
	    || ($url ==
		addSlash($_SESSION["conf-$config"]['sws']['baseurl']) . "index.php")
	    || isset($PAGE['options']['error'])) {
		return 0;
	}

	$backurl = substr($url, 0, strrpos($url, "/"));
	if (substr($url, -strlen("index.php")) == "index.php") {
		$backurl = substr($backurl, 0, strrpos($backurl, "/"));
	}
	$backurl .= "/index.php";
	return array($backurl, getURLTitle($backurl));
}

/**
 * Return the (topic, title) array for the given directory index
 * @param string $url the URL to get the title for
 * @return array the (topic, title) array for the given directory
 */
function getIndex($url)
{
	$url = normalizeURL($url);
	$url = substr($url, 0, strrpos($url, "/")) . "/index.php";
	return array($url, getURLTitle($url));
}

?>
