<?php

/**
 * .tar file creation functions.
 *
 * $Id: tar.php 493 2005-08-15 12:43:57Z stijn $
 * @package sws
 */

/**
 * Determine tar filesize based on file sizes + header sizes + extras.
 * @param string $dir the directory that will be archived
 * @return int the size of the tar file in bytes
 * @access private
 */
function getTarFileSize($dir = "")
{
	$s = stat($dir);
	$space = $s['blocks'] * 512;
	if (is_dir($dir)) {
		$dh = opendir($dir);
		while (($file = readdir($dh)) !== false) {
			if ($file != "." and $file != "..") {
				$tempsize = getTarFileSize($dir."/".$file);
				$tempsize += 512 - ($tempsize %512);		
				$space += $tempsize;
				if (strlen($file) > 100) {
					$space += 1024;
				} else {
					$space += 512;
				}
			}
		}
		closedir($dh);

	}
	return $space;
}

/**
 * Create the tar header for the specified file/directory
 * @param string $basedir the directory that will not be archived
 * @param string $tardir the directory to be archived
 * @param string $filename the name of the file to write a header for
 * @param string $linkflag pass "5" for WinZip compatibility
 * @return string the header for the archive
 * @access private
 */
function writeTarHeader($basedir = "", $tardir = "", $filename, $linkflag = "0")
{
	// forbidden character array to make names more winzip compatible
	$forbiddenchars = array("\\", ":", "*", "?", "\"", "<", ">", "|");

	// get the properties from the file
	$filelocation = ($basedir == "" ? "" : $basedir . "/") . 
			($tardir  == "" ? "" : $tardir  . "/") .
			$filename;
	$stat = stat($filelocation);

	// create a nice name
	$tempname = ($tardir  == "" ? "" : $tardir . "/") . $filename . "\0";
	$tempname = str_replace($forbiddenchars, "_", $tempname);
	
	// if the name is longer then 512 characters, fail miserably
	if (strlen($tempname) > 512) {
		exit(0);
	}	
	
	// if length larger then 100 first send an additional header
	$longfilename = "";
	if (strlen($tempname) > 100) {
		$name     = "././@LongLink";
		$name    .= str_repeat("\0", 100 - strlen($name));
		$mode     = "0000000\0";
		$uid      = "0000000\0";
		$gid      = "0000000\0";
		$size     = sprintf("%011o\0", strlen($tempname));
		$mtime    = "00000000000\0";
		$chksum   = "        ";
		$typeflag = "L";
		$linkname = str_repeat("\0", 100);
		$magic    = "ustar  \0";
		$version  = "";
		$uname    = "root";
		$uname   .= str_repeat("\0", 32 - strlen($uname));	
		$gname    = "wheel";
		$gname   .= str_repeat("\0", 32 - strlen($gname));
		$devmajor = str_repeat("\0", 8);
		$devminor = str_repeat("\0", 8);
		$prefix   = str_repeat("\0", 155);	

		$header = $name . $mode . $uid . $gid . $size . $mtime .
				$chksum . $typeflag . $linkname . $magic .
				$version . $uname . $gname . $devmajor .
				$devminor . $prefix . str_repeat("\0", 12);
		
		$sum = 0;
		for ($i = 0; $i < strlen($header); $i++) {
			$sum += ord($header[$i]);
		}
		$chksum = sprintf("%06o\0 ", $sum);
					 
		$longfilename = $name . $mode . $uid . $gid . $size . $mtime .
				$chksum . $typeflag . $linkname . $magic .
                                $version . $uname . $gname . $devmajor .
                                $devminor . $prefix . str_repeat("\0", 12) .
				$tempname . 
				str_repeat("\0", 512 - strlen($tempname)); 

		// cut the end of tempname to create a name with length 100
		// for the next part of the header
		$tempname = substr($tempname, 0, 99);
	}

	// extend the length op $name by filling it up with 
	// null characters
        $length = strlen($tempname);
	$name = $tempname . str_repeat("\0", 100 - $length);

	// format the stats into nice strings
	// put some hard coded entries into the header 
	// like magic, uname and gname
	$mode     = sprintf("%07o\0",  $stat['mode']);
	$uid      = sprintf("%07o\0",  $stat['uid']);
	$gid      = sprintf("%07o\0",  $stat['gid']);
	
	// XXX quick fix to be able to work with winzip
	if ($linkflag == "5") {
		$size = str_repeat("0", 11) . "\0";
	} else {
		$size     = sprintf("%011o\0", $stat['size']);
	}
	$mtime    = sprintf("%011o\0", $stat['mtime']);
	$chksum   = "        ";
	$typeflag = $linkflag;
	$linkname = str_repeat("\0", 100);
	$magic    = "ustar  \0";
	$version  = "";
	$uname    = posix_getpwuid($stat['uid']);
        $uname    = $uname['name'] .
                        str_repeat("\0", 32 - strlen($uname['name']));
	$gname    = posix_getgrgid($stat['gid']);
	$gname    = $gname['name'] . 
			str_repeat("\0", 32 - strlen($gname['name']));
	$devmajor = str_repeat("\0", 8);
	$devminor = str_repeat("\0", 8);
	$prefix   = str_repeat("\0", 155);

	// create the complete header with spaced out chksum
	// and calculate the chksum
	$header = $name . $mode . $uid . $gid . $size . $mtime . 
			$chksum . $typeflag . $linkname . $magic .
			$version . $uname . $gname . $devmajor . 
			$devminor . $prefix . str_repeat("\0", 12);
	$sum = 0;
	for ($i = 0; $i < strlen($header); $i++) {
		$sum += ord($header[$i]);
	}
	$chksum = sprintf("%06o\0 ", $sum);

	// create the new header with the updated information
        $header = $longfilename . $name . $mode . $uid . $gid . 
			$size . $mtime .  $chksum . $typeflag . 
			$linkname . $magic .  $version . $uname . 
			$gname . $devmajor .  $devminor . $prefix . 
			str_repeat("\0", 12);

 	return $header;
}

/**
 * Output one file for the archive.
 * @param string $basedir the directory that will not be archived
 * @param string $tardir the directory to be archived
 * @param string $file the name of the file to write a header for
 * @access private
 */
function writeFileToTarball($basedir, $tardir, $file = "")
{
	$path = $basedir . "/" . $tardir . "/" . $file;
	if (is_dir($path)) {
		$directory = $basedir . "/" . $tardir;
		$dirref = opendir($directory);
		
		// add the path to the header file
		print writeTarHeader($basedir, $tardir, "", "5");	

		// read every file from the directory apart from . and ..
		while (($file = readdir($dirref)) !== false) {
                	if (($file != ".") && ($file != "..")) {
				$path = $basedir . "/" .$tardir . "/" . $file;
                                
				if (is_dir($path)) {
					$newtardir = $tardir . "/" . $file;
					writeFileToTarball($basedir,
						$newtardir);
				} else {
					writeFileToTarball($basedir, 
						$tardir, $file);
				}
                        }
                }
		closedir($dirref);
        } else {
		// write the tar header
		print writeTarHeader($basedir, $tardir, $file);
		flush();

		// determine file size
		$size = filesize($path);
			
		// write all the data from the file
		if ($size != 0) { 
			$fileref = fopen($path, "r");
			while (!feof($fileref)) {
				print fread($fileref, 4096);
				flush();
			}
			fclose($fileref);
		
			// extend to fill up a complete block
			print str_repeat("\0", 512 - $size % 512);
			flush();
		}
	}
}

/**
 * Output a tar file.
 *
 * This will output all HTTP headers so that the browser will start a
 * download for a file of application/x-tar.
 * @param string $basedir the starting point where tar starts looking for the
 *                        directory to archive; it won't appear in the tarfile 
 * @param string $tardir the directory that will appear in the tarfile; all
 *                       the directories and files contained within it will be
 *                       added to the tarfile
 * @param string $filename the name that the .tar download will suggest
 */
function writeTarFile($basedir, $tardir, $filename = "")
{
	// forbidden character array to make names more winzip compatible
	$forbiddenchars = array("\\", ":", "*", "?", "\"", "<", ">", "|");
	
	// determine tar filename	
	$filename = str_replace($forbiddenchars, '', $filename);
	if ($filename == '')  {
		$namearray = explode("/", $tardir);
		$filename = end($namearray) . '.tar';	
	} else {
		$filename .= '.tar';
	}

	// determine tar file size
	$size = getTarFileSize($basedir . "/" . $tardir);
	
	// add the size for the 2 empty blocks
	$size += 2 * 512;

	/* prevents httpd going crazy */
	set_time_limit(0);
	ignore_user_abort(1);

	// send http headers
    header("Pragma: public");
    header("Expires: " . gmdate("D, d M Y H:i:s \G\M\T", time() + 3600));
    header("Last-Modified: " . gmdate("D, d M Y H:i:s") . " GMT");
    header("Cache-Control: no-store, max-age=0, no-cache, must-revalidate");
    header("Cache-Control: post-check=0, pre-check=0", false);
    header("Cache-control: private");
    header("Content-type: application/x-tar");
    header("Content-Disposition: attachment; filename=\"" . 
			$filename . "\"");
	header("Content-length: " . $size + 2 * 512);
	
	// write the separate files
	writeFileToTarball($basedir, $tardir);
	
	// write the end of the tar file, 2 empty blocks
	print str_repeat("\0", 2 * 512);
}

?>
