<?php

/**
 * Handles the procedure of logging in.
 *
 * $Id$
 * @package sws
 */

// First make sure we only validate on the login host. The signing won't
// work otherwise (at least in a secure setup).
if (!onLoginHost()) {
	redirectLogin("login.php?" .
	    (isset($_SERVER['QUERY_STRING']) ? $_SERVER['QUERY_STRING'] : ""));
	exit;
}

// Make sure we use https if it is available
encryptProtocol();

// This parameter gets set if we detect an absence of login related
// information in the request (no privs= parameter). It is used to determine
// whether the user has disabled their cookies.
if (isset($_GET['nocookie'])) {
	if (!isset($_COOKIE['nocookies'])) {
		needTheme();
		pre("Cookies disabled");
		print "<p>You must have cookies enabled to be able " .
		    "to login.</p>\n";
		post();
		exit;
	}

	unsetLoginCookie('nocookies');

	if (isset($_COOKIE['invalidlogin']) && $_COOKIE['invalidlogin'] == 2) {
		needTheme();
		pre("Login cookie invalid");
		print "<p>You have cookies enabled, but somehow your browser " .
		    "corrupts them, or the server cannot get valid " .
		    "information out of them. Please contact the " .
		    "server's administrator to get this fixed.</p>\n";
		if (ini_get("magic_quotes_gpc") == "1") {
			print "<p>The most likely cause of this error is " .
			    "the server setting of magic_quotes_gpc; it " .
			    "should be turned off.</p>\n";
		}
		post();
		exit;
	}

	// Either someone requested this file directly, or we got an
	// invalid login cookie the first time we were her.
	// Pretend that we were called for a generic login, and fall through.
	$_GET['privs'] = "";
}

// Check to see if this request included the desired privilege information.
// If it does, assume that this is the start of a new login process.
if (isset($_GET['privs'])) {
	$privs = $_GET['privs'];

	if (isset($_GET['back'])) {
		$back = $_GET['back'];
	} else {
		$back = defaultBackURL();
	}

	if (isset($_GET['reason']) && is_int((int)($_GET['reason'])) &&
	    ((int)($_GET['reason']) != 0)) {
		$reason = $_GET['reason'];
	} else {
		$reason = LOGIN_REQUIRED;
	}

	setLoginCookie('swslogin', serialize(array($privs, $back, $reason)));
	redirectLogin("login.php");
	exit;
}

// If we did not get request information (privs), we should have gotten a
// login cookie. This should be passed from the first request of this file
// and contains the information that we should verify. If it's not there,
// either the user cannot deal with cookies, or this page was requested
// directly (which should not happen). In order to distinguish the cases, set
// another cookie before redirecting back to ourselves. We will use this
// cookie above to print a message about cookies if it turns out the user
// disabled them.
if (!isset($_COOKIE['swslogin'])) {
	setLoginCookie('nocookies', 'false');
	redirectLogin("login.php?nocookie=true");
	exit;
}

// Import the private key and certificates.
$privid = openssl_get_privatekey("file://" . swsroot() . "/etc/keys/sws.key");
if ($privid === FALSE) {
	print "configuration error: private key file not found or unreadable";
	exit;
}

$pubid = openssl_get_publickey("file://" . swsroot() . "/etc/keys/sws.crt");
if ($pubid === FALSE) {
	print "configuration error: public key file not found or unreadable";
	exit;
}

// We have a cookie, check its contents for validity before continuing.
// Reset the login for invalid data.
$tmp = @unserialize($_COOKIE['swslogin']);
if ($tmp === FALSE || !is_array($tmp) ||
    !isset($tmp[0]) || !is_string($tmp[0]) ||
    !isset($tmp[1]) || !is_string($tmp[1]) ||
    !isset($tmp[2]) || !is_int((int)($tmp[2])) ||
    ((int)($tmp[2]) == 0)) {
	// We really have to reset the whole login; that means we would have to
	// redirect including the 'privs' and 'backlink' parameters. However
	// these have been lost at this point because of the mangling of the
	// login cookie. So we pretend that we were called directly, as above,
	// and redirect back to ourselves.
	// However, because there might be something wrong with the servers
	// handling of cookies in general (e.g. magic_quotes_gpc is set),
	// this could generate an infinite loop, where we keep redirecting
	// the user to the 'no cookies' code, and discovering there's nothing
	// wrong at their end.
	// The solution is to set yet another cookie that distinguishes the
	// 'invalid swslogin cookie' case from the "regular" direct request
	// case, and check that above. Because we do want to have the user
	// retry this case at least once, in case the browser somehow mangled
	// up the first cookie, we only abort the login process if that
	// cookie is at least up to try #2.
	unsetLoginCookie('swslogin');
	setLoginCookie('nocookies', 'false');
	if (isset($_COOKIE['invalidlogin'])) {
		setLoginCookie('invalidlogin', '2');
	} else {
		setLoginCookie('invalidlogin', '1');
	}
	redirectLogin("login.php?nocookie=true");
	exit;
}
unsetLoginCookie('invalidlogin');

// The login cookie has seemingly valid data, decode it.
$priv = $tmp[0];
$back = $tmp[1];
$reason = (int)$tmp[2];

// Sanity check the data
if ($priv == "") {
	$priv = array();
} else {
	$priv = explode(",", $priv);
}

if ($back == "") {
	$back = defaultBackURL();
}

// Now check for an existing ticket, and validate it. Reset all related
// data if it's not valid.
$tkt = array();
if (isset($_COOKIE['tkt'])) {
	$tkt = verifyTicket();
	if (!is_array($tkt)) {
		$tkt = array();
		unsetLoginCookie('tkt');
	}
}

// Get a privileged handle to the SWS database (to access auth data)
$db = db_get_sws_privileged_object();
if ($db === FALSE) {
	print "could not access privileged SWS database, configuration error.";
	exit;
}

// If we don't have a ticket, or don't have the right privileges, we
// prompt for a login.
if (!isset($tkt['id']) || !hasPriv($priv)) {
	// If no user information was posted, display a login form.
	if (!isset($_POST['user']) || !isset($_POST['password'])
	    || !matchReferrer()) {
		session_regenerate_id();
		needTheme();
		loginForm($reason);
		exit;
	}

	// We have form data. Validate the username.
	$user = $_POST['user'];
	$res = $db->query("SELECT id, password " .
	    "FROM users WHERE login = " . $db->quote($user));
	if (!$res || $res->num_rows() != 1) {
		logFailedLogin($db, $user, LOGIN_FAILED);
		needTheme();
		loginForm(LOGIN_FAILED);
		exit;
	}
	$row = $res->fetch_array();

	// Check to see if the account is administratively disabled
	$realpwd = $row['password'];
	if (!$realpwd || $realpwd == "") {
		logFailedLogin($db, $user, LOGIN_FAILED);
		needTheme();
		loginForm(LOGIN_FAILED);
		exit;
	}

	// Now check if the passwords match
	if (md5($_POST['password']) != $realpwd) {
		logFailedLogin($db, $user, LOGIN_FAILED);
		needTheme();
		loginForm(LOGIN_FAILED);
		exit;
	}

	// They matched, create a ticket
	$tkt['id'] = $row['id'];
	$tkt['login'] = $user;
	$tkt['useragent'] = md5($_SERVER['HTTP_USER_AGENT']);
	$tkt['remote_addr'] = remoteAddress();
	$tkt['start'] = date("Y-m-d H:i:s");
	$tkt['ts'] = time();

	$t = serialize($tkt);
	if (!openssl_sign($t, $s, $privid)) {
		logFailedLogin($db, $user);
		needTheme();
		loginForm(LOGIN_FAILED);
		exit;
	}

	// Record the login
	$res = $db->query("INSERT INTO wtmp (user_id, ip, start) VALUES (" .
	    $tkt['id'] . ", " . $db->quote($tkt['remote_addr']) . ", " .
	    $db->quote($tkt['start']) . ")");
	if (!$res) {
		logFailedLogin($db, $user);
		needTheme();
		loginForm(LOGIN_FAILED);
		exit;
	}

	// Now set the ticket cookie for further requests
	setLoginCookie('tkt', base64_encode($t) . "-" . base64_encode($s));

	// At this point, we have a valid ticket. Now check for requested
	// privileges ($priv), for it could be that the user succesfully
	// authenticated but still does not have enough privileges
	// (note that this is of course merely a shortcut).
	if (!hasPriv($priv)) {
		logFailedLogin($db, $user, LOGIN_PRIVS);
		needTheme();
		loginForm(LOGIN_PRIVS);
		exit;
	}
}

// We are cleared to go! Clean up, and then redirect to our saved back URL.
unsetLoginCookie('swslogin');
setLoginCookie('swsdone', 'login');
header("Location: " . $back);

?>
