<?php

/**
 * Default theme.
 *
 * This is mostly intended to be an example theme, to base new themes on.
 * $Id: index.php 597 2006-06-16 07:21:17Z stijn $
 * @package sws
 */

/**
 * Set options for the theme.
 *
 * Globally recognized theme options are:
 * - no_sidebar: if true, do not display a topics / boxes side bar
 * @param array $options hash of options and their values
 */
function setThemeOptions($options)
{
	global $PAGE;

	/* You can insert theme options into the $PAGE hash here. They can
	   then be modified by the page itself, or by the framework. */
	if (isset($options['encoding'])) {
		$PAGE['theme']['encoding'] = $options['encoding'];
	}
}

/**
 * Output all HTML needed before the content is output.
 *
 * The theme should use the $PAGE global variable to display stuff.
 * Important keys in the $PAGE variable:
 * - title: the title of the page
 * - options: the hash of options and their values
 * - topics: array of topics
 * - backlink: the link to the parent of this page
 */
function preContent()
{
	global $PAGE;

	pageHeader($PAGE["title"]);
	pageBoxStart($PAGE["index"][1], $PAGE["index"][0]);
	boxStart($PAGE["title"], "title");
}

/**
 * Output the HTML to be written after the content.
 *
 * Use the same values as for {@link preContent}.
 */
function postContent()
{
	global $PAGE;

	boxEnd();
	pageTopics($PAGE["topics"]);
	pageBoxEnd();
	pageFooter();
}

/**
 * Start a page element.
 *
 * Most themes define style to separate common page elements from each other.
 * This function should output the HTML necessary to do so.
 */
function boxStart($title = "", $class = "")
{
	if ($title == "") {
		$title = "Untitled";
	}

	print "<p" . ($class != "" ? " class=\"" . $class . "\"" : "") .
	    ">" . $title . "</p>\n";
}

/**
 * End a page element.
 *
 * This function should output the necessary HTML to end a page element.
 */
function boxEnd()
{
	print "<hr />\n";
}

/**
 * Output a login form.
 *
 * This should output a login form that sends POST information to the
 * current page. The form should have two fields: user and password.
 * The framework will call this when appropriate, and will handle the
 * resulting POST.
 *
 * It is expected that the display code uses the parameters passed to
 * this function to explain the reason for asking their credentials
 * to the user.
 * @param int $reason the reason for credential verification
 * @param string $db the name of the database to login to
 * @see LOGIN_REQUIRED
 * @see LOGIN_FAILED
 * @see LOGIN_PRIVS
 * @see LOGIN_EXPIRED
 * @see LOGIN_CONFIG
 */
function loginForm($reason, $db = NULL)
{
	global $PAGE;

	pageHeader("Login required");
	if (isset($PAGE["backlink"])) {
		pageBoxStart($PAGE["backlink"][1], $PAGE["backlink"][0]);
	} else {
		pageBoxStart("Login required", "");
	}
	boxStart("Login required");
        print "<form method=post action=\"" . me() . "\">\n";
	if ($reason == LOGIN_REQUIRED) {
		if (!isset($db)) {
	        	print "<p>Login required to view this page.</p>\n";
		} else {
			print "<p>Login required to access database $db.</p>\n";
		}
	} else if ($reason == LOGIN_FAILED) {
        	print "<p>Login failed, try again.</p>\n";
	} else if ($reason == LOGIN_PRIVS) {
		print "<p>Not enough privileges, try again.</p>\n";
	} else if ($reason == LOGIN_EXPIRED) {
		print "<p>Your login has expired, please relogin.</p>\n";
	} else if ($reason == LOGIN_CONFIG) {
		print "<p>Error in login configuration, warn administrator.</p>\n";
	}
        print "<table border=0>\n";
        print "<tr>\n";
        print "  <td>Login:</td>" .
                "<td><input type=\"text\" name=\"user\"></td>\n";
        print "</tr>\n";
        print "<tr>\n";
        print "  <td>Password:</td>" .
                "<td><input type=\"password\" name=\"password\"></td>\n";
        print "</tr>\n";
        print "<tr>\n";
        print "  <td>&nbsp;</td>" .
                "<td><input type=\"submit\" value=\"Login\"></td>\n";
        print "</tr>\n";
        print "</table>\n";
        print "</form>\n";
	boxEnd();
	pageBoxEnd();
	pageFooter();
}

/*
 * The functions from here on are all internal functions.
 */

/**
 * Display the start of the content page element.
 * @access private
 */
function pageBoxStart($title, $url)
{
}

/**
 * Display the end of the content page element.
 * @access private
 */
function pageBoxEnd()
{
	print "<div style=\"text-align: right\">&copy; " . date("Y") .
	    "</div>\n";
}

/**
 * Display the header for the page.
 * @param string $title the title for the page.
 * @access private
 */
function pageHeader($title)
{
	global $PAGE;

	if (isset($PAGE['theme']['encoding']) && !headers_sent()) {
		header("Content-Type: text/html; charset=" .
		    $PAGE['theme']['encoding']);
	}
	print "<?xml version=\"1.0\" encoding=\"" .
	    (isset($PAGE['theme']['encoding']) ? $PAGE['theme']['encoding'] :
		"ISO-8859-1") . "\"?>\n";
	print "<!DOCTYPE html\n" .
	    " PUBLIC \"-//W3C//DTD XHTML 1.0 Strict//EN\"\n" .
	    "\"http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd\">\n";
	print "<html xmlns=\"http://www.w3.org/1999/xhtml\" xml:lang=\"en\" " .
	    "lang=\"en\">\n";
	print "<head>\n";
	if (isset($PAGE['theme']['encoding'])) {
		print "<meta http-equiv=\"Content-Type\" " .
		    "content=\"text/html; charset=" .
		    $PAGE['theme']['encoding'] . "\" />\n";
	}
	print "  <title>" . $title . "</title>\n";
	print "</head>\n";
	print "<body>\n";
}

/**
 * Display the footer for the page.
 * @access private
 */
function pageFooter()
{
	print "</body>\n";
	print "</html>\n";
}

/**
 * Display the topics for a page
 * @param array $topics the topics array (as returned by {@link getTopics} or
 *                      {@link convertTopics}
 * @access private
 */
function pageTopics($topics)
{
	global $PAGE;

	if ($topics != 0) {
		print "<div style=\"font-size: small\">\n";
		print "<p>Other topics in this directory:</p>\n";
		print "<ul>\n";
		for ($i = 0; $i < sizeof($topics); $i++) {
			print "<li><a href=\"" . $topics[$i][0] . "\">"
				. $topics[$i][1] .  "</a></li>\n";
		}
		print "</ul>\n";
		print "</div>\n";
	}
	if (isset($PAGE["backlink"][0])) {
		print "<div style=\"font-size: small\">Back to " .
		    "<a href=\"" . $PAGE["backlink"][0] . "\">" .
		    $PAGE["backlink"][1] . "</a></div>\n";
	}
}

?>
