#
# $Id: Update.pm 398 2004-04-19 09:16:12Z stijn $
#
# A perl script to create and or update the SWS database structure.
#
# For every database change, include a new subroutine that
# updates the specified database to the new version. Call this subroutine
# 'update_to_${VERSION}' and it will automatically be called by the
# main db_update loop if necessary. Be sure to call set_version to
# update the version of the database.
#

package SWS::Update;

use strict;
use vars qw($VERSION @ISA @EXPORT @EXPORT_OK);

require Exporter;

@ISA = qw(Exporter);
@EXPORT = qw(
	add_column drop_column rename_column
	add_table drop_table rename_table
	add_index
	set_version db_update
);
$VERSION = '0.5';

use DBI;
use SWS::Error;

my $dbh;

sub add_column
{
	my ($table, $name, $type, $after) = @_;

	$dbh->do("ALTER TABLE $table ADD COLUMN $name $type" .
		(defined($after) ? " AFTER $after" : "")) || 
	    die("error while adding column $table.$name");
}

sub drop_column
{
	my ($table, $name) = @_;

	$dbh->do("ALTER TABLE $table DROP $name") || 
	    die("error while adding column $table.$name");
}

sub rename_column
{
	my ($table, $old, $new, $type) = @_;

	add_column($table, $new, $type, $old);
	$dbh->do("UPDATE $table SET $new = $old") ||
	    die("error while copying $table.$old to $table.$new");
	drop_column($table, $old);
}

sub add_table
{
	my ($name, $def) = @_;
	$dbh->do("CREATE TABLE $name ($def)") ||
	    die("error while creating table $name");
}

sub drop_table
{
	my ($table) = @_;
	$dbh->do("DROP TABLE $table") ||
	    die("error while dropping table $table");
}

sub rename_table
{
	my ($old, $new) = @_;
	$dbh->do("ALTER TABLE $old RENAME $new") ||
	    die("error while renaming table $old to $new");
}

sub add_index
{
	my ($name, $on) = @_;
        $dbh->do("CREATE INDEX $name ON $on") ||
	    die("error while creating index $name");
}

sub set_version
{
	my ($name, $version) = @_;

	die "set_version: need name and version"
	    if (!defined($name) || !defined($version));

	$dbh->do("UPDATE dbparm SET " . $name . "version = " . $version)
		|| die("could not set version to $version for $name");
}

sub db_update
{
	$dbh = $_[0];
	my $targetversion = $_[1];
	my $name = $_[2];
	my $dbversion = -1;

	my $result = $dbh->selectall_arrayref(
	    "SELECT " . $name . "version FROM dbparm");
	if (!defined($result) || scalar(@{$result}) != 1) {
		print("==> Couldn't determine " . $name . " database " .
		    "version - assuming new installation\n");
		if (defined($DBI::errstr) &&
		    $DBI::errstr =~ /does.*exist/) {
			add_table("dbparm",
			    $name . "version	int not null default 0");
			$dbh->do("INSERT INTO dbparm (" . $name . "version) " .
			    "VALUES (0)") ||
				die("could not set initial " . $name .
				    "version");
		} else {
			add_column("dbparm", $name . "version", "int");
		}
		$dbversion = 0;
	} else {
		$dbversion = @{$result}[0]->[0];
	}

	if ($dbversion < $targetversion) {
		print "==> upgrading from version $dbversion to " .
		    "version $targetversion\n";
		while ($dbversion < $targetversion) {
			eval("&main::update_to_" . ($dbversion + 1));
			$dbversion = $dbversion + 1;
		}
	}
}

1;
